/***
BEGIN LICENSE
Copyright (C) 2011 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/
using GLib;
using Gee;
using Sqlite;

namespace Foto.AlbumManager {

    public class AlbumHelper{

        public ArrayList<Album> AlbumList;
        private Database db;
        private string database_dir = "/Foto.db";
        private string old_database_dir = "/FotoAlbum.sql";
        public signal void album_created (string album_name);   // this signal is emited when a new album is created
        public signal void album_updated (string album_name);

        private static string insert_album = "INSERT INTO Album VALUES ('%s', '%s', '%s', '%s', '%d');";
        private static string insert_picture = "INSERT INTO Picture VALUES ('%s', '%s', '%s');";
        private static string delete_album_query = "DELETE FROM Album WHERE album_name = '%s';";
        private static string delete_picture_by_path = "DELETE FROM Picture WHERE picture_path = '%s';";
        private static string delete_picture_by_album = "DELETE FROM Picture WHERE album = '%s';";
        private static string update_album_query ="UPDATE Album SET album_name = '%s', 
                                                             			description = '%s', 
                                                             			album_cover_path = '%s',
                                                                        rating = %d
                                                             	 WHERE album_name = '%s';";
        /*private static string update_picture_query ="UPDATE Picture SET picture_path = '%s', 
                                                                 	album = '%s', 
                                                                 	description = '%s' 
                                                                 WHERE picture_path = '%s';";*/

        private static string update_picture_album_query ="UPDATE Picture SET album = '%s' WHERE album = '%s';";

        private static string create_table_album = "CREATE TABLE Album(album_name TEXT primary key not null,description TEXT,
                                                                          creation_time date not null,album_cover_path TEXT not null, 
                                                                          rating INT not null);";

        private static string create_table_picture = "CREATE TABLE Picture(picture_path TEXT primary key not null,album TEXT,
                                                                              description TEXT,FOREIGN KEY(album) REFERENCES Album(album_name));";
        private static string select_album = "SELECT * FROM Album ORDER BY album_name;";
        private static string select_picture = "SELECT * FROM Picture WHERE album = '%s' ORDER BY picture_path;";

        //constructor
        public AlbumHelper(){

            string DATA_DIR = Environment.get_user_data_dir();
            string db_location_foto = DATA_DIR + "/foto";
            Granite.Services.Paths.initialize("foto", Build.PKGDATADIR);
            database_dir = db_location_foto + database_dir;
            old_database_dir = db_location_foto + old_database_dir;

            AlbumList = new ArrayList<Album>();

            if (!FileUtils.test (database_dir, FileTest.IS_REGULAR)) {
                debug("Database %s does not exist, creating new database\n", database_dir);
                create_album_db();
            }
            else{
                debug("Database %s already exists, loading albums\n", database_dir);
                populate_album_list();
            }
        }

        public static int callback (int n_columns, string[] values,
                                      string[] column_names){

            for (int i = 0; i < n_columns; i++) {
                stdout.printf ("%s = %s\n", column_names[i], values[i]);
            }
            stdout.printf ("\n");
            return 0;
        }

        //an album exists if there is another album with the same name
        public bool album_exists(Album new_album){

            if(AlbumList.size<=0) return false;

            foreach(Album album in AlbumList){
                if(album.album_name == new_album.album_name) return true;
            }

            return false;
        }

        public bool album_name_exists(string new_album){

            if(AlbumList.size<=0) return false;

            foreach(Album album in AlbumList){
                if(album.album_name == new_album) return true;
            }
            return false;
        }
        
        public Album? get_album_by_name(string album_name){
        	foreach(Album album in AlbumList){
        		if(album.album_name == album_name)
        			return album;
        	}
        	return null;
        }
        

        //create db
        private void create_album_db(){

            if(FileUtils.test (old_database_dir, FileTest.IS_REGULAR)) {
                FileUtils.unlink (old_database_dir);
                debug("Old database has been deleted");
            }

            //create database
            int rc = Database.open_v2 (database_dir, out db);
            if (rc != Sqlite.OK)
                critical("Couldn't open database: %d, %s", rc, db.errmsg ());

            //create table album
            rc = db.exec (create_table_album, (Sqlite.Callback) callback, null);
            if (rc != Sqlite.OK)
                error("SQL error in '%s': %d, %s\n", create_table_album, rc, db.errmsg ());

            //create table picture
            rc = db.exec (create_table_picture, (Sqlite.Callback) callback, null);
            if (rc != Sqlite.OK)
                error("SQL error in '%s': %d, %s\n", create_table_picture, rc, db.errmsg ());
        }

        //populate album list from db
        private void populate_album_list(){
            //open database
            int rc = Database.open_v2 (database_dir, out db);
            if (rc != Sqlite.OK)
                critical("Couldn't open database: %d, %s", rc, db.errmsg ());
            Statement stmt;
            if ((rc = db.prepare_v2 (select_album, -1, out stmt, null)) == 1) {
                printerr ("SQL error: %d, %s\n", rc, db.errmsg ());
                return;
             }

            string album_name=null, description=null, creation_time=null, album_cover=null;
            int rating = 3;
            int cols = stmt.column_count(), col;
            do {
                rc = stmt.step();
                switch (rc) {
                case Sqlite.DONE:
                    break;
                case Sqlite.ROW:
                    for (col = 0; col < cols; col++) {
                        
                        switch(col){
                            case 0:
                                album_name = stmt.column_text(col);
                                break;
                            case 1:
                                description = stmt.column_text(col);
                                break;
                            case 2:
                                creation_time = stmt.column_text(col);
                                break;
                            case 3:
                                album_cover = stmt.column_text(col);
                                break;
                            case 4:
                                rating = stmt.column_int(col);
                                break;
                            default:
                                break;
                        }
                    }
                    //TODO: Load album_cover
                    parse_datetime(creation_time);
                    var new_album = new Album(album_name,description,parse_datetime(creation_time), null, rating);
                    AlbumList.add(new_album);
                    populate_picture_list(new_album);
                    break;
                default:
                    printerr ("Error: %d, %s\n", rc, db.errmsg ());
                    break;
                }
            } while (rc == Sqlite.ROW);
        }


        private void populate_picture_list(Album album){

            Statement stmt;
            int rc;
            if ((rc = db.prepare_v2 (select_picture.printf(album.album_name), -1, out stmt, null)) == 1) {
                printerr ("SQL error: %d, %s\n", rc, db.errmsg ());
                return;
             }

            string picture_path="", album_name = null, description=null;
            int cols = stmt.column_count(), col;
            do {
                rc = stmt.step();
                switch (rc) {
                case Sqlite.DONE:
                    break;
                case Sqlite.ROW:
                    for (col = 0; col < cols; col++) {
                        string txt = stmt.column_text(col);
                        switch(col){
                            case 0:
                                picture_path = txt;
                                break;
                            case 1:
                                album_name = txt;
                                break;
                            case 2:
                                description = txt;
                                break;
                            default:
                                break;
                        }
                    }
                    album.add_picture(picture_path, description);
                    break;
                default:
                    printerr ("Error: %d, %s\n", rc, db.errmsg ());
                    break;
                }
            } while (rc == Sqlite.ROW);
        }



        //create new album (insert and then add to album list).
        public bool create_album(Album album){
            //if album already exists, we can't add it
           if(album_exists(album)) return false;

           string picture_path = "";
           if(album.album_cover != null) picture_path = album.album_cover.picture_path;

           db.exec (insert_album.printf(album.album_name, 
                                        album.description, 
                                        album.get_creation_time(), 
                                        picture_path,
                                        album.rating), 
                                        (Sqlite.Callback) callback, null);
            AlbumList.add(album);
            album_created (album.album_name);
            return true;
        }

        public bool delete_album(Album album){
            //delete the album
            db.exec (delete_album_query.printf(album.album_name), (Sqlite.Callback) callback, null);
            //delete all pics in album
            db.exec (delete_picture_by_album.printf(album.album_name), (Sqlite.Callback) callback, null);
            AlbumList.remove(album);
            return true;
        }


        //TODO: ALL
        public bool update_album(string album_name, Album edited_album){
        
        	var album = get_album_by_name(album_name);
        	string picture_path;
        	
        	if (album == null) return false;
        	warning("Editing album");
        	
            picture_path = edited_album.album_cover.picture_path;
            if(picture_path == null) picture_path = "";
        
        	db.exec (update_album_query.printf(edited_album.album_name, edited_album.description, 
        									   picture_path, edited_album.rating, album_name),
        									(Sqlite.Callback) callback, null);
        	
            if(album.album_name != edited_album.album_name){
        	    db.exec (update_picture_album_query.printf(edited_album.album_name, album.album_name),
        									              (Sqlite.Callback) callback, null);
            }
								
        	album.album_name = edited_album.album_name;
        	album.description = edited_album.description;
            album.rating = edited_album.rating;
        	
        	album_updated(album.album_name);
            return false;
        }

        public bool create_picture(string picture_path, string album_name, string description){
            db.exec (insert_picture.printf(picture_path, album_name, description), (Sqlite.Callback) callback, null);
            foreach(Album album in AlbumList){
                if(album.album_name == album_name){
                    album.add_picture(picture_path, description);
                    break;
                }
            }
        	album_updated(album_name);
            return false;
        }

        //TODO: ALL
        public bool delete_picture(Picture picture){
            db.exec (delete_picture_by_path.printf(picture.picture_path), (Sqlite.Callback) callback, null);
            return false;
        }
        //TODO: ALL
        public bool update_picture(Picture picture){
            return false;
        }

        /*parse a string in YYYY-MM-DDTHH:MM:SS-SSS format
        * returning a DateTime
        */
        private GLib.DateTime? parse_datetime(string time){
            string year="";
            string day="";
            string month="";
            string hour="";
            string minute="";
            string seconds="";
            long limit=0;
         
            //parse year
            for(long i=0;i<time.char_count();i++){
                limit = i;
                if(time.get(i) == '-') break;
                year = year + time.substring (i,1);
            }
            limit++;
            //parse month
            for(long i=limit;i<time.char_count();i++){
                limit = i;
                if(time.get(i) == '-') break;
                month = month + time.substring (i,1);
            }
            limit++;
            //parse day
            for(long i=limit;i<time.char_count();i++){
                limit = i;
                if(time.get(i) == 'T') break;
                day = day + time.substring (i,1);
            }
            limit++;
            //parse hour
            for(long i=limit;i<time.char_count();i++){
                limit = i;
                if(time.get(i) == ':') break;
                hour = hour + time.substring (i,1);
            }
            limit++;
            //parse minute
            for(long i=limit;i<time.char_count();i++){
                limit = i;
                if(time.get(i) == ':') break;
                minute = minute + time.substring (i,1);
            }
            limit++;
            //parse seconds
            for(long i=limit;i<time.char_count();i++){
                if(time.get(i) == '-') break; //we are not interested in timezone (for now)
                seconds = seconds + time.substring (i,1);
            }

            //now we create the datetime
            int d_year,d_day,d_month,d_hour,d_minute;
            double d_seconds;
            d_year = int.parse(year);
            d_day = int.parse(day);
            d_month = int.parse(month);
            d_hour = int.parse(hour);
            d_minute = int.parse(minute);
            d_seconds = double.parse(seconds);

            var date = new DateTime.local(d_year,d_month,d_day,d_hour,d_minute,d_seconds);
            return date;
        }

    }     
}

