/***
BEGIN LICENSE
Copyright (C) 2011 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/

using Gtk;
using Gdk;
using Gee;
using GtkClutter;
using Clutter;
using Granite.Widgets;
using Granite.Services;
using Foto.Views;
using Foto.Widgets;
using Foto.AlbumManager;
using Foto.Settings;

namespace Foto {

    //for esier access
    public static AlbumHelper album_manager;
    public static SavedState saved_state;
    public static MainWindow main_window;
    public static FotoSettings settings;
    public FotoApp fotoapp;        

    public class FotoApp : Granite.Application {

            // Chrome widgets
	        public static Granite.Widgets.AppMenu appmenu;
	        public static FotoMenu menu;
	        public static string[] _args;
            public static Utils.IconFactory icon_factory = new Utils.IconFactory();
            //Saved state

        construct {
	        set_flags (ApplicationFlags.HANDLES_OPEN);
            //this.register(); 
            //FIXME: this cause a segmention fault when closing app
            saved_state = new SavedState ();
            settings = new FotoSettings ();   
            //use dark theme                    
            if(settings.use_dark_theme)
            	Gtk.Settings.get_default ().gtk_application_prefer_dark_theme = true;
           // App info
            build_data_dir = Build.DATADIR;
            build_pkg_data_dir = Build.PKGDATADIR;
            build_release_name = Build.RELEASE_NAME;
            build_version = Build.VERSION;
            build_version_info = Build.VERSION_INFO;

            program_name = "Foto";
            exec_name = "foto";

            app_years = "2012-";
            application_id = "net.launchpad.foto";
            app_icon = "foto";
            app_launcher = "foto.desktop";

            main_url = "https://launchpad.net/foto";
            bug_url = "https://bugs.launchpad.net/foto";
            help_url = "https://answers.launchpad.net/foto";
            translate_url = "https://translations.launchpad.net/foto";

		    about_authors = {"Erasmo Marin <erasmo.marin@gmail.com>"};
		    about_artists = {"Camilo Highita", "Erasmo Marín <erasmo.marin@gmail.comma>"};
            about_translators = "";
            about_license_type = License.GPL_3_0;
            about_documenters = {
                "Erasmo Marín <erasmo.marin@gmail.com>"
            };

            about_translators = "Launchpad translators";
            about_license_type = Gtk.License.GPL_3_0;
        }

        public FotoApp() {
		    activate();
       }
        
        protected override void open (File[] files, string hint) {
            foreach(File file in files)
                print("\n"+file.get_path());
            activate ();
        }

        protected override void activate () {
            Granite.Services.Logger.DisplayLevel = Granite.Services.LogLevel.DEBUG;
		    if (get_windows () == null) {
			    //get_windows ().data.present (); // present window if app is already running
                album_manager = new AlbumHelper ();
	            menu = new FotoMenu ();
	            appmenu = create_appmenu (menu);
	            main_window = new MainWindow (appmenu,menu, _args);
		        menu.set_new_window (main_window);

                main_window.key_press_event.connect ((e) => {

                    switch (e.keyval) {
                        case Gdk.Key.@q:
                        case Gdk.Key.@w:
                            if ((e.state & Gdk.ModifierType.CONTROL_MASK) != 0) {
                                main_window.destroy ();
                            }
                            break;
                        }
                    return false;
                });

            //supported formats
            /*
            SList<PixbufFormat> list = Gdk.Pixbuf.get_formats();

            foreach(PixbufFormat format in list){
                warning(format.get_name());
            }*/
                Gtk.main ();
            
            } else {
                warning("New instance");
            }

        }

        public static int main (string[] args) {                        
            var err = GtkClutter.init (ref args);
            if (err != Clutter.InitError.SUCCESS) {
                error ("Could not initalize clutter! "+err.to_string ());
            }
            Clutter.init (ref args);
            _args = args;
            fotoapp = new FotoApp ();
            return fotoapp.run (args);
        }
    }
}
