//
//  Copyright (C) 2012 Erasmo Marín
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using GExiv2;
using GLib;

namespace MetadataReader{

    /**
    *   Wrapper class for GExiv2. 
    *   It makes easier to deal with metadata.
    *   It also read basic metadata from the file
    *   with FileInfo
    */
    public class PictureMetadata{

        private Metadata metadata;
        private FileInfo fileinfo;
        private string image_path;

        public string mime_type { owned get { return metadata.get_mime_type (); } }

        public string camera_model { owned get { return metadata.get_camera_model (); } }
        public string comment { owned get { return metadata.get_comment (); } }
        public string date_time { owned get { return metadata.get_date_time (); } }
        public string[] tags { owned get { return metadata.get_exif_tags (); } }
        public double focal_length { get { return metadata.get_focal_length (); } }

        public int pixel_width { get { return metadata.get_pixel_width (); } }
        public int pixel_height { get { return metadata.get_pixel_height (); } }

        public GExiv2.Orientation orientation {
            set{
                metadata.set_orientation(value);
                save();
            } 
            get { return metadata.get_orientation (); } 
        }

        public uint rating {
            set{
                if(value<=5) {
                    metadata.set_rating(value);
                    save();
                }
                else
                    warning("The value must be less than or equal to 5");
            } 
            get { return metadata.get_rating(); }
        }


        public string last_access; //TIME_ACCESS
        public string last_changed; //TIME_CHANGED
        public string creation_time; //TIME_CREATED
        public string last_modified; //TIME_MODIFIED

        public int64 size;

        public PictureMetadata(string image_path){

            metadata = new Metadata();
            File file = File.new_for_path(image_path);

            try {
                metadata.open_path(image_path);
                fileinfo = file.query_info(FileAttribute.STANDARD_NAME, FileQueryInfoFlags.NONE);  
            } catch (Error e) {
                warning (e.message);
            }

            this.image_path = image_path;           

            size = fileinfo.get_size();

            /*NOT WORKING
            last_access = fileinfo.get_attribute_as_string (FileAttribute.TIME_ACCESS);
            last_changed = fileinfo.get_attribute_as_string (FileAttribute.TIME_CHANGED);
            creation_time = fileinfo.get_attribute_as_string (FileAttribute.TIME_CREATED);
            last_modified = fileinfo.get_attribute_as_string (FileAttribute.TIME_MODIFIED);
            */
        }

        private void save(){
            try {
                metadata.save_file(image_path);
            } catch (Error e) {
                warning (e.message);
            }
        }


        public string orientation_to_string (GExiv2.Orientation orientation){
            switch(orientation){
                case GExiv2.Orientation.UNSPECIFIED:
                    return "Unknown";
                case GExiv2.Orientation.VFLIP:
                    return "Vertical Flip";
                case GExiv2.Orientation.HFLIP:
                    return "Horizontal Flip";
                case GExiv2.Orientation.ROT_90_HFLIP:
                    return "90º Horizontal Flip";
                case GExiv2.Orientation.ROT_90_VFLIP:
                    return "90º Vertical Flip";
                case GExiv2.Orientation.ROT_90:
                    return "90º Rotation";
                case GExiv2.Orientation.ROT_180:
                    return "180º Rotation";
                case GExiv2.Orientation.ROT_270:
                    return "270º Rotation";
                case GExiv2.Orientation.NORMAL:
                    return "Normal";
                default:/*FIXME: MIN MAX NOT HANDLED*/
                    return "Unknown";
            }
        }
    }
}
