//  Copyright (C) 2012-2013 Erasmo Marín <erasmo.marin@gmail.com>
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
using Gdk;
using Gdk;
using GLib;

//TODO: Save cache images async
namespace Utils{

    public class Cache {

        static string cache_folder = null;

        private static void init () {
            if (cache_folder == null)
                cache_folder = Environment.get_user_cache_dir () + "/foto/";
        }

        /*create a new cache for image path*/
        public static bool cache_image (string image_path, int width, int height) {
            try {
                var pixbuf = new Pixbuf.from_file_at_scale (image_path, width, height, true);
                debug ("Image cached: " + get_cache_path () + compute_key (image_path));
                pixbuf.save (get_cache_path () + compute_key (image_path) , "png");
            } catch (GLib.Error err) {
     	        stdout.printf ("%s\n", err.message);
                return false;
            }
            return true;
        }

        /*Determine if a image is cached*/
        public static bool is_cached (string image_path) {
            File file = File.new_for_path (get_cache_path () + compute_key (image_path));
            if (!file.query_exists ())
                return false;
            return true;
        }

        /*this returns the cached thumbnail.**/
        public static Pixbuf? get_cached_image (string image_path) {

            string computed_key = compute_key (image_path);
            try {
                var pixbuf = new Pixbuf.from_file (get_cache_path () + computed_key);
                return pixbuf;
            } catch (GLib.Error err) {
     	       stdout.printf ("%s\n", err.message);
                return null;
            }     
        }

        //FIXME
        public async static Pixbuf? get_cached_image_async (string image_path) {
            string computed_key = compute_key (image_path);
            Pixbuf image;
            GLib.File file = GLib.File.new_for_commandline_arg (get_cache_path () + computed_key);
            try {
	            GLib.InputStream stream = yield file.read_async ();
	            image = yield new Pixbuf.from_stream_async (stream, null);
                return image;
            } catch (GLib.Error err) {
     	       stdout.printf ("%s\n", err.message);
                return null;
            }
        }

        public static async void empty_cache () {
            debug ("Start deleting cache\n");
            var dir = File.new_for_path (cache_folder);
            try {
                // asynchronous call, to get directory entries
                var e = yield dir.enumerate_children_async (FileAttribute.STANDARD_NAME,
                                                        0, Priority.DEFAULT);
                while (true) {
                    // asynchronous call, to get entries so far
                    var files = yield e.next_files_async (10, Priority.DEFAULT);
                    if (files == null) {
                        break;
                    }
                    // append the files found so far to the list
                    foreach (var info in files) {
                        var file = dir.get_child (info.get_name ());
                        file.delete();
                    }
                }
            } catch (Error err) {
                stderr.printf ("Error: list_files failed: %s\n", err.message);
            }
        }


        /*
        * Compute the key from an image path
        */
        private static string compute_key (string image_path) {
            return Checksum.compute_for_string (ChecksumType.MD5, image_path);
        }

        private static string get_cache_path () {
            Cache.init ();
            return cache_folder;
        }
    }
}
