//  Copyright (C) 2012 Erasmo Marín <erasmo.marin@gmail.com>
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
using Gdk;
using GLib;

namespace Utils{

    public class IconFactory {

        public enum IconType {

            ICON_CLOSE_BUTTON,
            ICON_EDIT_BUTTON,
            ICON_NO_PICTURE,
            ICON_IMAGE_MISSING,

            STAR_ACTIVE,
            STAR_INACTIVE,
            STAR_PRELIGHT,/*Still not implemented*/

            STAR_ACTIVE_SYMBOLIC,
            STAR_INACTIVE_SYMBOLIC,
            STAR_PRELIGHT_SYMBOLIC /*Still not implemented*/

        }


        private static Gdk.Pixbuf ICON_CLOSE_BUTTON;
        private static Gdk.Pixbuf ICON_EDIT_BUTTON;
        private static Gdk.Pixbuf ICON_NO_PICTURE;
        private static Gdk.Pixbuf ICON_IMAGE_MISSING;

        private static Gdk.Pixbuf STAR_ACTIVE;
        private static Gdk.Pixbuf STAR_INACTIVE;
        private static Gdk.Pixbuf STAR_PRELIGHT;

        private static Gdk.Pixbuf STAR_ACTIVE_SYMBOLIC;
        private static Gdk.Pixbuf STAR_INACTIVE_SYMBOLIC;
        private static Gdk.Pixbuf STAR_PRELIGHT_SYMBOLIC;

        public IconFactory() {
        	try {
                ICON_CLOSE_BUTTON = new Gdk.Pixbuf.from_file(Build.PKGDATADIR + "/close_button.png");
                ICON_EDIT_BUTTON = new Gdk.Pixbuf.from_file(Build.PKGDATADIR + "/edit_button.png");
                ICON_NO_PICTURE = new Gdk.Pixbuf.from_file(Build.PKGDATADIR + "/no_picture.png");
                ICON_IMAGE_MISSING = new Gdk.Pixbuf.from_file(Build.PKGDATADIR + "/image_missing.png");

                STAR_ACTIVE = new Gdk.Pixbuf.from_file(Build.PKGDATADIR + "/star_active.png");
                STAR_INACTIVE = new Gdk.Pixbuf.from_file(Build.PKGDATADIR + "/star_inactive.png");
                STAR_PRELIGHT = STAR_INACTIVE;

                STAR_ACTIVE_SYMBOLIC = new Gdk.Pixbuf.from_file(Build.PKGDATADIR + "/star_active_symbolic.png");
                STAR_INACTIVE_SYMBOLIC = new Gdk.Pixbuf.from_file(Build.PKGDATADIR + "/star_inactive_symbolic.png");
                STAR_PRELIGHT_SYMBOLIC = STAR_INACTIVE;

			} catch (GLib.Error error) {
                 warning("There was a problem loading icons.");
		    }


        }

        //static method to get icons
        public static Gdk.Pixbuf? get_icon(IconType icon_type) {

                switch (icon_type) {
                    case IconType.ICON_CLOSE_BUTTON:
                        return ICON_CLOSE_BUTTON;

                    case IconType.ICON_EDIT_BUTTON:
                        return ICON_EDIT_BUTTON;

                    case IconType.ICON_NO_PICTURE:
                        return ICON_NO_PICTURE;

                    case IconType.ICON_IMAGE_MISSING:
                        return ICON_IMAGE_MISSING;

                    case IconType.STAR_ACTIVE:
                        return STAR_ACTIVE;

                    case IconType.STAR_INACTIVE:
                        return STAR_INACTIVE;

                    case IconType.STAR_PRELIGHT:
                        return STAR_PRELIGHT;

                    case IconType.STAR_ACTIVE_SYMBOLIC:
                        return STAR_ACTIVE_SYMBOLIC;

                    case IconType.STAR_INACTIVE_SYMBOLIC:
                        return STAR_INACTIVE_SYMBOLIC;

                    case IconType.STAR_PRELIGHT_SYMBOLIC:
                        return STAR_PRELIGHT_SYMBOLIC;

                    default:
                        assert_not_reached();
                }
        }



    }
}
