//  Copyright (C) 2012-2013 Erasmo Marín <erasmo.marin@gmail.com>
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
using Gdk;
using Granite.Drawing;

//TODO: Save cache images async
namespace Utils{

    public class PixbufUtils {

        public static Gdk.Pixbuf? render_box_shadow (Gdk.Pixbuf pixbuf,
                                                  int surface_size_w, int surface_size_h,
                                                  int shadow_size = 5, double alpha = 0.75) {

            int S_WIDTH = (surface_size_w > 0)? surface_size_w : pixbuf.width;
            int S_HEIGHT = (surface_size_h > 0)? surface_size_h : pixbuf.height;

            var buffer_surface = new Granite.Drawing.BufferSurface (S_WIDTH, S_HEIGHT);

            S_WIDTH -= 2 * shadow_size;
            S_HEIGHT -= 2 * shadow_size;

            buffer_surface.context.rectangle (shadow_size, shadow_size, S_WIDTH, S_HEIGHT);
            buffer_surface.context.set_source_rgba (0, 0, 0, alpha);
            buffer_surface.context.fill ();

            buffer_surface.fast_blur (2, 3);

            Gdk.cairo_set_source_pixbuf (buffer_surface.context,
                                         pixbuf.scale_simple (S_WIDTH, S_HEIGHT, Gdk.InterpType.BILINEAR),
                                         shadow_size,
                                         shadow_size);

            buffer_surface.context.paint ();

            return buffer_surface.load_to_pixbuf ();
        }

        public static Gdk.Pixbuf? render_drop_shadow (Pixbuf source, int shadow_size, double alpha_threshold, Granite.Drawing.Color shadow_color) {

            if(!source.has_alpha) return render_box_shadow (source, source.width,source.height);

            var copy = source.copy();

            var a = 0.0;
            
            uint8* dataPtr = copy.get_pixels ();
            double pixels = copy.height * copy.rowstride / copy.n_channels;
            
            for (var i = 0; i < pixels; i++) {
                a = dataPtr [3];

                if(a >= alpha_threshold) {
                    dataPtr[0] = (uint8) shadow_color.B;
                    dataPtr[1] = (uint8) shadow_color.G;
                    dataPtr[2] = (uint8) shadow_color.R;
                    dataPtr[3] = (uint8) shadow_color.A;
                } else {
                    dataPtr[0] = 0;
                    dataPtr[1] = 0;
                    dataPtr[2] = 0;
                    dataPtr[3] = 0;
                }
                dataPtr += copy.n_channels;
            }

            var buffer_surface = new Granite.Drawing.BufferSurface (copy.width, copy.height);
            copy = copy.scale_simple (copy.width-2*shadow_size, copy.height-2*shadow_size, Gdk.InterpType.HYPER);
            Gdk.cairo_set_source_pixbuf (buffer_surface.context, copy, shadow_size, shadow_size);
            buffer_surface.context.paint ();
            buffer_surface.fast_blur (shadow_size, 3);
            Gdk.cairo_set_source_pixbuf (buffer_surface.context, 
                                         source.scale_simple (source.width-2*shadow_size, source.height-2*shadow_size, Gdk.InterpType.BILINEAR), 
                                         shadow_size, shadow_size);
            buffer_surface.context.paint ();
            return buffer_surface.load_to_pixbuf ();
        }

    }
}
