/***
  BEGIN LICENSE
	
  Copyright (C) 2011 Mario Guerriero <mefrio.g@gmail.com>
		        2012 Erasmo Marín <erasmo.marin@gmail.com>	
  This program is free software: you can redistribute it and/or modify it	
  under the terms of the GNU Lesser General Public License version 3, as published	
  by the Free Software Foundation.
	
  This program is distributed in the hope that it will be useful, but	
  WITHOUT ANY WARRANTY; without even the implied warranties of	
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR	
  PURPOSE.  See the GNU General Public License for more details.
	
  You should have received a copy of the GNU General Public License along	
  with this program.  If not, see <http://www.gnu.org/licenses/>	
  
  END LICENSE	
***/ 

using Gtk;
using Gee;
using Foto.AlbumManager;

// Helper class for the file IO functions we'll need
// Not needed at all, but helpful for organization

namespace Utils{

public class IOHelper : GLib.Object {

    // Check if the filename has a picture file extension.
    public static bool is_valid_file_type (string fname) {

        // Cache a lowe-cased copy of the file name
        string fname_down = fname.down();
        // Short-circuit if it's not a picture file extension.
        return (fname_down.has_suffix(".png")  || 
                 fname_down.has_suffix(".jpeg") || 
                 fname_down.has_suffix(".jpg")  || 
                 fname_down.has_suffix(".gif")  || 
                 fname_down.has_suffix(".svg")  || 
                 fname_down.has_suffix(".bmp")  ||
                 fname_down.has_suffix(".tiff") ||
                 fname_down.has_suffix(".tif")  ||
                 //fname_down.has_suffix(".xbm")  ||
                 //fname_down.has_suffix(".tga")  ||
                 //fname_down.has_suffix(".pcx")  ||
                 //fname_down.has_suffix(".wmf")  ||
                 //fname_down.has_suffix(".ras")  ||
                 //fname_down.has_suffix(".wbmp") ||
                 //fname_down.has_suffix(".pnm")  ||
                 fname_down.has_suffix(".ico")
                 //fname_down.has_suffix(".icns") ||
                 //fname_down.has_suffix(".qtif") ||
                 //fname_down.has_suffix(".ani")  ||
                 //fname_down.has_suffix(".xpm")
                 );
    }

    // Quickly count up all of the valid images in the images folder.
    public static int count_images (GLib.File image_folder) {
        GLib.FileInfo file_info = null;
        int count = 0;
        try {
            // Get an enumerator for all of the plain old files in the wallpaper folder.
            var enumerator = image_folder.enumerate_children(FileAttribute.STANDARD_NAME + "," 
								+ FileAttribute.STANDARD_TYPE, 0);
            // While there's still files left to count
            while ((file_info = enumerator.next_file ()) != null) {
                // If it's a picture file
                if (file_info.get_file_type() == GLib.FileType.REGULAR && is_valid_file_type(file_info.get_name() ) ) {
                    count++;
                }
            }
        } catch(GLib.Error err) {
            // Just for debugging sake
            stdout.printf("Could not pre-scan folder. Progress percentage may be off: %s\n", err.message);
        }
        return count;
    }
	
	public static string get_file_name(string full_filename){

		string[] name = full_filename.split("/");
		string current_filename = null;
		foreach (var s in name) current_filename = s;
		return current_filename;
	}

	public static string get_file_folder(string full_filename){
		string[] name;
		name = full_filename.split("/" + get_file_name(full_filename));
		return name[0];
	}
}

public class ImageList : ArrayList<string> {

	int last_query_index;
	string folder_str;
	File folder;

	public ImageList.from_folder(string folder_str /*Without final slash*/){
		last_query_index = -1;
		this.folder_str = folder_str;
		folder = File.new_for_path(folder_str);
    	        GLib.FileInfo file_info = null;

      	try {
     	    // Get an enumerator for all of the plain old files in the image folder.
     	    var enumerator = folder.enumerate_children(FileAttribute.STANDARD_NAME + "," 
						   + FileAttribute.STANDARD_TYPE, 0);
     	    while ((file_info = enumerator.next_file ()) != null) {
     	       // If it's a picture file
     	       if (file_info.get_file_type() == GLib.FileType.REGULAR && 
			       IOHelper.is_valid_file_type(file_info.get_name())) {
			       //add the file name to the image list
     	           this.add(file_info.get_name());
     	       }
     	    }
            sort_alphabetically();
        } catch(GLib.Error err) {
     	       stdout.printf("%s\n", err.message);
        }
	}


    public ImageList.from_album (Album album) { 
        this.clear();
        folder_str = null;
        foreach (Picture pic in album.picture_list) {
            this.add(pic.picture_path);
        }
        this.sort_alphabetically();
    }

    public void sort_alphabetically() {
        sort_with_data(((CompareDataFunc)GLib.strcmp));
    }

	public string get_filename(int index){
		if(index>0 && index< this.size){
			last_query_index = index;
			return this.get(index);
		}
		else return "";
	}

	public string get_full_filename(int index) {
		if(get_filename(index)!=null)
            if(folder_str == null) return this.get_filename(index);
		    else return folder_str + "/" + this.get_filename(index);
		else return "";
	}

	public void clean(){
		this.clear();
	}
}
}








