/***
BEGIN LICENSE
Copyright (C) 2011 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/

using Gdk;
using Gtk;
using Gee;
using Foto.Widgets;
using Foto.AlbumManager;
using Utils;


namespace Foto.Views{

public class FotoAlbumGrid : DynamicGrid{

    protected ArrayList<Gtk.Widget> widgets;
    protected ArrayList<Gtk.Widget> grid_filter;
    private EmbeddedAlert alert_view;
	
    public signal void files_dragged(File[] files);

	public FotoAlbumGrid (ArrayList<Album> album_list) {
        base (180);
        support_drag_and_drop ();
        alert_view = new EmbeddedAlert ();
        alert_view.set_vexpand (true);
        alert_view.set_alert (_("No albums found"), _("Try changing your search terms"), null, true, Gtk.MessageType.INFO);
        load_album_grid_list (album_list);
        album_manager.album_created.connect (attach_album);
       	album_manager.album_updated.connect (update_albums);
        this.show_all ();
	}

    private void attach_album () {        
        var album = album_manager.AlbumList.get (album_manager.AlbumList.size-1);
        var album_widget = new AlbumWidget.from_album (album,150,150);
        attach_new_widget (album_widget);
    }

    private void load_album_grid_list (ArrayList<Album> album_list) {
        grid_list.add (new AddButton (WIDGET_SIZE, WIDGET_SIZE));
        if (album_list.size != 0) {
            foreach (Album album in album_list) {
                grid_list.add (new AlbumWidget.from_album (album, WIDGET_SIZE, WIDGET_SIZE));
            }
        }
        widgets = grid_list;
        load_widgets ();        
    }

    public void attach_new_widget (Widget widget) {
        COLS = calcule_columns ();
        grid_list.insert (grid_list.size-1, widget);
        remove_widgets ();
        load_widgets ();
        this.show_all ();
    }
    
    public void filter_albums (string search) {
    	//null search
    	if (search == "") {
            remove_widgets();
            grid_list = widgets;
            load_widgets();
        	return;
    	}
    	//search
        var filter = new ArrayList<Gtk.Widget>();
    	foreach (Widget album in widgets) {
    		AlbumWidget aux;
    		if (album is AlbumWidget) {
    			aux = (AlbumWidget) album;
    			if(aux.album_name.down ().contains (search.down())) {
                    filter.add(aux);
                }
    		}
    	}
        remove_widgets();
        if (filter.size == 0) {
            filter.add(alert_view);
        }
        grid_list = filter;
        load_widgets();
    }

    public void on_album_destroy (Widget widget) {
        grid_list.remove (widget);
        remove_widgets ();
        COLS = calcule_columns ();
        load_widgets ();    
    }
    
    public void update_albums () {
    	foreach (Widget widget in grid_list) {
    		AlbumWidget album;
    		if (widget is AlbumWidget) {
    			album = (AlbumWidget) widget;
    			album.update_labels ();
    		}   		
    	}
    }

    private void support_drag_and_drop () {

        Gtk.drag_dest_set (this, DestDefaults.ALL, {}, Gdk.DragAction.COPY);
        Gtk.drag_dest_add_uri_targets (this);
        drag_data_received.connect ((dc, x, y, selection_data, info, time) => {
            //Foto app = Foto.get_instance ();

            File[] files = {};
            foreach (string uri in selection_data.get_uris ()) {
                if (0 < uri.length){
                    var new_file = File.new_for_uri (uri);
                    if (IOHelper.is_valid_file_type (new_file.get_path() )) {
                        files += new_file;
                        print("\nFile dragged: " + new_file.get_path());
                    }
                }
            }
            //app.open_documents (files);
            Gtk.drag_finish (dc, true, true, time);
            files_dragged(files);
        });
    }

}
}
