/***
BEGIN LICENSE
Copyright (C) 2012 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/

using Gdk;
using Gtk;
using Gee;

namespace Foto.Views{

public class DynamicGrid : Gtk.ScrolledWindow{

    //grid where we put the widgets
	protected Gtk.Grid content_grid;
	private Gtk.Viewport viewport;
    protected ArrayList<Gtk.Widget> grid_list;
    //the widget size (it's only important for width)
    protected int WIDGET_SIZE = 180;
    //columns number, by default, is 5
    protected int COLS = 5;

	public DynamicGrid(int widget_size){
        shadow_type = ShadowType.NONE;
        WIDGET_SIZE = widget_size;
        grid_list = new ArrayList<Gtk.Widget>();
		content_grid = create_grid();
		viewport = new Gtk.Viewport(null,null);

        var style_provider = Utils.Css.get_css_provider ();
        content_grid.get_style_context ().add_provider (style_provider, Gtk.STYLE_PROVIDER_PRIORITY_THEME);
        content_grid.get_style_context ().add_class ("grid");
		viewport.add(content_grid);
	    this.set_border_width (0);
		this.add(viewport);
        viewport.set_shadow_type (Gtk.ShadowType.NONE);
        this.size_allocate.connect(on_resize);
        this.set_policy(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
        this.set_vexpand(true);
        content_grid.draw.connect(draw_background);
	}

    //we re draw the content grid
    public void on_resize(){
        int new_cols = calcule_columns();
        if(new_cols == COLS  || new_cols==0 || new_cols > grid_list.size) return;
        COLS = new_cols;
        remove_widgets();
        load_widgets();
    }

    public void remove_widgets() {

        if(grid_list.size == 0) return;
        foreach (Widget widget in grid_list) {
            	content_grid.remove(widget);
        }
    }

    public void clean_widgets(){
        if(grid_list.size == 0) return;
        foreach (Widget widget in grid_list){
            	content_grid.remove(widget);
                widget.destroy();
        }
        grid_list.clear();
    }

    public void load_widgets () {

        int i,j;
        bool flag=false;

        if(grid_list.size == 0) return;
        for(i=0;;i++){
            for(j=0;(j<COLS && (i*COLS+j)<=grid_list.size);j++){
                if((i*COLS+j)>=grid_list.size) {
                    flag=true; 
                    break;
                }
                content_grid.attach(grid_list.get(i * COLS + j), j, i, 1, 1);
                grid_list.get(i * COLS + j).show();
            }
            if(flag) break;
        }
        this.show_all();
    }

    protected int calcule_columns () {
        int cols = (int)(this.get_allocated_width()/(WIDGET_SIZE + content_grid.column_spacing));
        if (cols > grid_list.size) cols = grid_list.size;
        return cols;
    }

    protected int calcule_max_columns() {
        return (int)(this.get_allocated_width()/(WIDGET_SIZE + content_grid.column_spacing));
    }

   	public void add_widget(Widget widget){
        grid_list.add(widget);
    }

    private Gtk.Grid create_grid(){
        var grid = new Gtk.Grid();
	    grid.column_spacing = 5;
	    grid.row_spacing = 5;
    	grid.column_homogeneous = true;
	    grid.row_homogeneous = false;
        grid.set_vexpand(true);
        grid.set_hexpand(true);
	    grid.set_border_width (0);
        return grid;
    }

    //draw a nice background
    private bool draw_background (Cairo.Context cr) {

        int width = content_grid.get_allocated_width ();
        int height = content_grid.get_allocated_height ();

		cr.set_operator (Cairo.Operator.CLEAR);
		cr.paint ();
		cr.set_operator (Cairo.Operator.OVER);
			
        var background_style = content_grid.get_style_context ();
		background_style.render_background (cr, 0, 0, width, height);
		background_style.render_frame (cr, 0, 0, width, height);

		var pat = new Cairo.Pattern.for_surface (new Cairo.ImageSurface.from_png (Build.PKGDATADIR + "/files/texture.png"));
		pat.set_extend (Cairo.Extend.REPEAT);
		cr.set_source (pat);
		cr.paint_with_alpha (0.6);
			
		return false;
	}
}
}
