/***
BEGIN LICENSE
Copyright (C) 2012 Erasmo Marín <erasmo.marin@gmail.com> 
based on ViewContainer from Noise project
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/
using Gdk;
using Gtk;
using Gee;
using Foto.Widgets;

namespace Foto.Views{

public enum ViewState{
    IMAGE_VIEWER=0,
    ALBUM_VIEWER=1,
    IMAGEGRID_VIEWER=2;

    public int to_int() {
        switch (this) {
            case IMAGE_VIEWER:
                return 0;

            case ALBUM_VIEWER:
                return 1;

            case IMAGEGRID_VIEWER:
                return 2;

            default:
                return 0;
        }
    }
    
    public ViewState? copy(){
    
    	ViewState new_state = ViewState.ALBUM_VIEWER;
    
        switch (this) {
            case IMAGE_VIEWER:
            	new_state = ViewState.IMAGE_VIEWER;
				break;
				
            case ALBUM_VIEWER:
                new_state = ViewState.ALBUM_VIEWER;
				break;
				
            case IMAGEGRID_VIEWER:
            	new_state = ViewState.IMAGEGRID_VIEWER;
				break;
				
            default:
            	break;
        }
       return new_state;  
    }
}


public class FotoViewContainer : Gtk.Notebook {

    ViewState state {set;get;}
    //Views
	public FotoAlbumGrid album_grid;
	public FotoViewer fotoviewer;
	public ImageGridView image_grid;
    private MainWindow window;

	public signal void view_state_changed ();

    public FotoViewContainer (string[] _args /*The app args*/, MainWindow window) {
        // Hide notebook tabs and border
        this.window = window;
        show_tabs = false;
	    set_border_width (0);
        show_border = false;
        //views
	    album_grid = new FotoAlbumGrid (album_manager.AlbumList);
	   	image_grid = new ImageGridView ();
        //var clutter = create_animatable_view(album_grid);

		fotoviewer = new FotoViewer(_args[1]);
	    add_view (fotoviewer);

        add_view (album_grid);
       	add_view (image_grid);


	    if(_args[1] != null) {
            set_view_state(ViewState.IMAGE_VIEWER);
            window.title = _args[1] + " - Foto";
        }
	    else 
            set_view_state(ViewState.ALBUM_VIEWER);

        fotoviewer.image_changed.connect( (image_basename) => {
                window.title = image_basename + " - Foto";
        });
    }

    /**
     * Appends a widget to the main views.
     *
     * @return the index of the view in the view container
     */
    public int add_view (Gtk.Widget? view) {
        if (view == null)
            return -1;
        return append_page (view);
    }


    public GtkClutter.Embed create_animatable_view(Gtk.Widget container){
        GtkClutter.Embed clutter;
        GtkClutter.Actor actor;

        clutter = new GtkClutter.Embed ();
        actor = new GtkClutter.Actor ();
        actor.scale_gravity = Clutter.Gravity.CENTER;
        actor.add_constraint (new Clutter.BindConstraint (clutter.get_stage (), 
            Clutter.BindCoordinate.WIDTH, 0));
        actor.add_constraint (new Clutter.BindConstraint (clutter.get_stage (), 
            Clutter.BindCoordinate.HEIGHT, 0));
        (actor.get_widget () as Gtk.Container).add (container);
        clutter.get_stage ().add_child (actor);
        //clutter.scroll_event.connect(()=>close_view_animation(clutter, actor));
        //clutter.button_press_event.connect(()=>open_view_animation(clutter, actor));
        return clutter;
    }

    /*private bool close_view_animation(GtkClutter.Embed clutter, GtkClutter.Actor actor) {
            actor.animate (Clutter.AnimationMode.EASE_IN_CUBIC, 300, scale_x : 2.0f, scale_y : 2.0f, opacity : 0).
                completed.connect (() => {
                //clutter.hide ();
            });
            warning("animation");
            return true;
    }

    private bool open_view_animation(GtkClutter.Embed clutter, GtkClutter.Actor actor) {
            clutter.show ();
            actor.animate (Clutter.AnimationMode.EASE_IN_CUBIC, 250, scale_x : 1.0f, scale_y : 1.0f, opacity : 255);
            return true;
    }*/



    /**
     * Removes a widget from the main views.
     *
     * @return the index of the view in the view container
     */
    public void remove_view (Gtk.Widget? view) {
        if (view == null)
            return;
        remove_page (get_view_index (view));
    }

    public Gtk.Widget? get_view (int index) {
        return get_nth_page (index);
    }

    public int get_view_index (Gtk.Widget? view) {
        if (view == null)
            return -1;
        return page_num (view);
    }

    public Gtk.Widget? get_current_view () {
        return get_view (get_current_index ());
    }

    public int get_current_index () {
        return get_current_page ();
    }

    /**
     * Tries to set the given view as current.
     * @return false if fails. true if succeeds
     */
    public bool set_current_view (Gtk.Widget? view) {
        if (view == null)
            return false;

        int index = get_view_index (view);

       // GtkNotebooks don't show hidden widgets.
       // Let's show the view just in case ...
        view.show ();

        return set_current_view_from_index (index);
    }

    /**
     * Tries to set the given view index as current.
     * @return false if fails. true if succeeds
     */
    public bool set_current_view_from_index (int index) {
        if (index < 0) {
            critical ("Cannot set " + get_view (index).name + " as the active view");
            return false;
        }
        set_current_page (index);

        return true;
    }

    public void set_view_state(ViewState new_state){
        state = new_state;
        set_current_view_from_index(state.to_int());
        view_state_changed();
        if(state != ViewState.IMAGE_VIEWER)
            window.set_title("Foto");
        this.show_all();
    }

    public ViewState get_view_state(){
        return state;
    }
    
    
    public void search(string search){
    	if(state == ViewState.ALBUM_VIEWER){
    		album_grid.filter_albums(search);
    	}
    }

}

}
