/***
BEGIN LICENSE
Copyright (C) 2012 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/

using Gdk;
using Gtk;
using Gee;
using Foto.Widgets;
using Foto.AlbumManager;
using Foto.Dialogs;
using Utils;


namespace Foto.Views{

public class ImageGridView : Gtk.Box{
	
    ArrayList<Picture> picture_list;
    Album album;
    //ProgressDialog progress_dialog;
    DynamicGrid grid;
    AlbumInfo info;
    DropArea drop_area;
    double import_progress = 0;

	public ImageGridView(){
        GLib.Object (orientation: Gtk.Orientation.VERTICAL);
        grid = new DynamicGrid(150);
        info = new AlbumInfo();
        drop_area = new DropArea(_("This album is empty!"),_("Drag some pics here"));
        drop_area.files_dropped.connect((files) => on_drop(files));
        support_drag_and_drop ();
        var separator = new CoolSeparator(Gtk.Orientation.HORIZONTAL);
        this.pack_start (info,false,false,0);
        this.pack_start (separator,false,false,0);
        this.pack_end (drop_area, true, true, 0);
        this.pack_end (grid,true,true,0);
        this.show_all();
	}

    public void set_album(Album album){
        if(album == this.album)
            return;
        this.album = album;
        info.set_album(album);
        grid.clean_widgets();
        this.picture_list = album.picture_list;

        if (picture_list == null || picture_list.size == 0) {
            this.remove(grid);
            this.pack_end (drop_area, true, true, 0); 
        }
        else {
            this.pack_end (grid,true,true,0);
            this.remove (drop_area);
        }

        for(int i=0;i<picture_list.size;i++){
       	    var icon = new Foto.Widgets.Icon.from_image_file(picture_list.get(i).picture_path, 150, 150);
            icon.set_have_close_button(false);
            icon.set_have_edit_button(false);
            icon.icon_clicked_event.connect( () => {on_icon_click (icon.cover_filename);});
       	    grid.add_widget(icon);
        }
            grid.load_widgets();
    }

    private void on_icon_click(string filename) {
        main_window.view_container.fotoviewer.set_from_album(album);
        main_window.view_container.fotoviewer.draw_from_filename(filename);
        while (Gtk.events_pending()) {Gtk.main_iteration(); }
        main_window.view_container.set_view_state(ViewState.IMAGE_VIEWER);
        while (Gtk.events_pending()) {Gtk.main_iteration(); }
    }


    private void on_drop (File[] files) {

        this.pack_end (grid,true,true,0);
        this.remove (drop_area);

        foreach(File file in files){
            if (IOHelper.is_valid_file_type (file.get_path() )) {
                message("File dragged: " + file.get_path());
                album_manager.create_picture(file.get_path(), album.album_name, "");
                var icon = new Foto.Widgets.Icon.from_image_file(file.get_path(), 150, 150);
                icon.set_have_close_button(false);
                icon.set_have_edit_button(false);
                icon.icon_clicked_event.connect( () => {on_icon_click (icon.cover_filename);});
       	        grid.add_widget(icon);
                grid.load_widgets();
                while (Gtk.events_pending()) {Gtk.main_iteration(); }
            }
        }
    }

    private void support_drag_and_drop () {

        Gtk.drag_dest_set (this.grid, DestDefaults.ALL, {}, Gdk.DragAction.COPY);
        Gtk.drag_dest_add_uri_targets (this.grid);
        grid.drag_data_received.connect ((dc, x, y, selection_data, info, time) => {
            int pics=0;
            File[] files = {};
            foreach (string uri in selection_data.get_uris ()) {
                if (0 < uri.length){
                    var new_file = File.new_for_uri (uri);
                    if (IOHelper.is_valid_file_type (new_file.get_path() )) {
                        files += new_file;
                        pics++;
                        message("File dragged: " + new_file.get_path());
                    }
                }
            }
            Gtk.drag_finish (dc, true, true, time);
            import_progress = 0;
            //progress_dialog = new ProgressDialog(_("Importing your pics"), _("Please wait while your pictures are imported."));

            foreach(File file in files){
                album_manager.create_picture(file.get_path(), album.album_name, "");
                var icon = new Foto.Widgets.Icon.from_image_file(file.get_path(), 150, 150);
                icon.set_have_close_button(false);
                icon.set_have_edit_button(false);
                icon.icon_clicked_event.connect( () => {on_icon_click (icon.cover_filename);});
       	        grid.add_widget(icon);
                grid.load_widgets();
                while (Gtk.events_pending()) {Gtk.main_iteration(); }
                import_progress = 1/pics;
                pics--;
            }
        });
    }


}
    //a dark and cool separator
    private class CoolSeparator : Gtk.DrawingArea {
        

        public CoolSeparator (Gtk.Orientation Orientation) {

            if (Orientation == Gtk.Orientation.HORIZONTAL) {

                this.set_vexpand(false);
                this.set_hexpand(true);
                this.height_request = 2;

            } else {

                this.set_vexpand(true);
                this.set_hexpand(false);
                this.width_request = 2;

            } 
        }

        //TODO: HORIZONTAL and VERTICAL separator
        public override bool draw (Cairo.Context cr){

            cr.set_source_rgba (0, 0, 0, 0.7);
            cr.move_to (0, 1);
            cr.line_to (get_allocated_width(), 1);
            cr.stroke ();

            cr.set_source_rgba (0, 0, 0, 0.71);
            cr.move_to (0, 2);
            cr.line_to (get_allocated_width(), 2);
            cr.stroke ();
            return false;
        }


    }


}
