/***
BEGIN LICENSE
Copyright (C) 2012 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/

using Gee;
using Utils;
using Foto.Dialogs;
using Foto.Widgets;
using Foto.Settings;
using Foto.AlbumManager;

namespace Foto.Views{

public class FotoViewer : Gtk.Box{

    private PictureContainer fotoviewer;
    private ArrayList<string> images;
	private string current_file_path = null;
	private string current_filename = null;
	private FotoviewerPopupMenu popupmenu;
	public FotoThumbnailPane thumbnails;
	public bool thumbnails_are_visible = true;
    public signal void image_changed(string filepath);


	//It use a filename string like "/folder/to/image.jpg"
	public FotoViewer(string? filename){
        set_spacing (0);
        set_border_width (0);

        images = new ArrayList<string>();

        if(filename != null) {
	        this.current_filename = filename;
		    current_file_path = IOHelper.get_file_folder(this.current_filename);
		    var image_list = new ImageList.from_folder(IOHelper.get_file_folder(this.current_filename));
            foreach (string base_name in image_list) {
                images.add(current_file_path + "/" + base_name);
                print("Added " + current_file_path + "/" + base_name + "\n");
            }
        }

		this.events |= Gdk.EventMask.BUTTON_RELEASE_MASK;
        this.button_release_event.connect (on_button_release);

		fotoviewer = new PictureContainer(this.current_filename);
		this.pack_start(fotoviewer, true, true, 0);
		this.show_all();

		popupmenu = new FotoviewerPopupMenu(this);
	}

    public void set_from_album(Album album) {

        images.clear();

        foreach (Picture pic in album.picture_list) {
            images.add(pic.picture_path);
        }
    }


	public ArrayList<string> get_imagelist(){
		return images;
	}

	public async void draw_next(){

        int index = images.index_of(current_filename);
        if(index == (images.size-1)) index = 0;
        else index++;
        current_filename = images.get(index);
        fotoviewer.viewer.set_image(current_filename);
        image_changed(current_filename);
	}

	//Draw the previous image in the image list
	public async void draw_prev(){

        int index = images.index_of(current_filename);
        if (index == 0) index = images.size-1;
        else index--;
        current_filename = images.get(index);
        fotoviewer.viewer.set_image(current_filename);
        image_changed(current_filename);
	}

	public async void draw_from_filename(string filename){
        current_filename = filename;
        fotoviewer.viewer.set_image(current_filename);
        image_changed(current_filename);
	}


	//Set the current image as background
	public async void set_as_background () {
		    GLib.Settings settings = new GLib.Settings ("org.gnome.desktop.background");
		    settings.set_string ("picture-uri", "file://" + current_filename);
	}

    private bool on_button_release (Gdk.EventButton event) {
	    //if the right click was released, we show the popup
	    if(event.button == 3) popupmenu.show_popup();
	    return true;
	}

	//remove the thumbnails from the box
	public void remove_thumbnails(){
		thumbnails_are_visible = false;
		stdout.printf("Hiding thumbnails");
		remove(thumbnails);
        FotoSettings conf = new FotoSettings();
        conf.show_thumbnails_pane = false;
	}

	public void show_thumbnails(){
		thumbnails_are_visible = true;
		stdout.printf("Showing thumbnails");
		pack_start(thumbnails, false, false, 0);
        FotoSettings conf = new FotoSettings();
        conf.show_thumbnails_pane = true;
	}

	public string get_file_path(){
		return current_file_path;
	}

	public string get_filename(){
		return current_filename;
	}


}
	//Pop up menu that is show when the user right click the image
	public class FotoviewerPopupMenu : Gtk.Menu {

	private FotoViewer fotoviewer;
    public Gtk.MenuItem set_wall;
    public Gtk.MenuItem view_properties;
	public Gtk.MenuItem show_menu;
	public Gtk.Menu show_submenu;
	public Gtk.CheckMenuItem show_thumbnail_pane;
	public Gtk.CheckMenuItem show_status_bar;

		public FotoviewerPopupMenu(FotoViewer fotoviewer){
			//set up menu items
			this.fotoviewer = fotoviewer;
			this.set_wall = new Gtk.MenuItem.with_label(_("Set as background"));
			this.view_properties = new Gtk.MenuItem.with_label(_("Properties"));

			//this.show_thumbnail_pane = new Gtk.CheckMenuItem.with_label (_("Show Thumbnail pane"));
			this.show_status_bar = new Gtk.CheckMenuItem.with_label (_("Status bar"));
            FotoSettings conf = new FotoSettings();            
			this.show_thumbnail_pane.set_active(conf.show_thumbnails_pane);
	

			//appends
			append(set_wall);
			//append (show_thumbnail_pane);
			append(view_properties);

			//signals
			set_wall.activate.connect(this.fotoviewer.set_as_background);
			view_properties.activate.connect(this.show_properties);
			//show_thumbnail_pane.toggled.connect(show_thumbnails);

			this.show_all();
		}

		//Draw the popup at the mouse position
		public void show_popup(){
			popup(null, null, null, 2, 0);
		}

		//Show a new window with the properties of the image file
		public void show_properties(){
			new PropertiesWindow(fotoviewer.get_filename());
		}

		public void show_thumbnails(){

			if(!show_thumbnail_pane.get_active()){
				this.fotoviewer.remove_thumbnails();
			}
			else{
				this.fotoviewer.show_thumbnails();
			}
		}

    }
}
