//
//  Copyright (C) 2012 Erasmo Marín
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
using Gdk;
using Gtk;
using Cairo;
using Granite.Drawing;
using Pango;
using Foto.AlbumManager;
using Foto.Dialogs;
using Foto.Views;


namespace Foto.Widgets{

private class AddButton :  DrawingArea{

    private int center_value=20;
    private int margin_x=10;
    private int margin_y=0;
    private int cross_size = 30;
    public int width;
    public int height;
    //style
    public RGBA line_color;
    public RGBA line_color_prelight;
    public RGBA icon_line_color;

    public AddButton(int width, int height){
            line_color = {1.0, 1.0, 1.0, 0.55};
            line_color_prelight = {1.0, 1.0, 1.0, 0.70};
            icon_line_color = {1.0, 1.0, 1.0, 0.95};
            set_size_request(width,height);
            this.width = width;
            this.height = height;
            this.set_tooltip_text(_("Add new album"));
            add_events (Gdk.EventMask.BUTTON_PRESS_MASK
                        | Gdk.EventMask.BUTTON_RELEASE_MASK 
                        | Gdk.EventMask.LEAVE_NOTIFY_MASK 
                        | Gdk.EventMask.ENTER_NOTIFY_MASK );

            set_vexpand(false);
            set_hexpand(false);
    }

    /* Widget is asked to draw itself */
    public override bool draw (Cairo.Context cr){
            margin_x = (this.get_allocated_width() - width)/2;
            //draw rectangle
            if(get_state_flags() == StateFlags.PRELIGHT)
                cr.set_source_rgba (1.0, 1.0, 1.0, 0.70);
            else
                cr.set_source_rgba (1.0, 1.0, 1.0, 0.55);
            Utilities.cairo_rounded_rectangle (cr, margin_x + center_value/2, margin_y + center_value/2,width - center_value,height - center_value, 5);
            //very hard to find
            cr.set_dash (new double[] {(137.999)/19,(137.95)/15.905},6);
            cr.set_line_width(4);
            cr.stroke ();
            cr.set_dash (null, 0);
            //draw + button, first vertical line
             cr.set_line_width(10);
             if(get_state_flags() == StateFlags.PRELIGHT)
                cr.set_source_rgba (1.0, 1.0, 1.0, 0.75);
            cr.move_to (width/2 + margin_x, height/2 - cross_size + margin_y);
            cr.line_to (width/2 + margin_x, height/2 + cross_size + margin_y);
            //now horizontal line
            cr.move_to (width/2 - cross_size + margin_x, height/2 + margin_y);
            cr.line_to (width/2 + cross_size + margin_x, height/2 + margin_y);
            cr.stroke ();       

            return false;
    }

    /* Mouse button got pressed over widget */
    public override bool button_press_event (Gdk.EventButton event) {
        this.set_state_flags(StateFlags.FOCUSED,true);
        return false;
    }

    /* Mouse button got released */
    public override bool button_release_event (Gdk.EventButton event) {
        this.set_state_flags(StateFlags.PRELIGHT,true);
        new AlbumDialog(null);
        return false;
    }

    public override bool enter_notify_event (Gdk.EventCrossing event) {
        this.set_state_flags(StateFlags.PRELIGHT,true);
        return false;
    }

    public override bool leave_notify_event (EventCrossing event){
        this.set_state_flags(StateFlags.NORMAL,true);
        return false;
    }
}
}
