/***
BEGIN LICENSE
Copyright (C) 2012 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/

using Gtk;
using Granite.Widgets;
using Foto.Widgets;
using Foto.AlbumManager;

namespace Foto.Widgets{

    public class AlbumInfo : Gtk.EventBox{

        Album album;
        DateFormat date_format = DateFormat.YYYYDDMM;
        Foto.Widgets.Icon icon;
        Rating rating;
        WrapLabel description_view;
        LLabel album_name_label;
        Gtk.Box mainbox;
        public string label_color = "#ffffff";

        public AlbumInfo () {
            //size
            set_vexpand(false);
            height_request = 130;

            //icon box
            var icon_box = new Gtk.Box(Orientation.VERTICAL, 0);
            icon_box.set_hexpand(false);
            icon = new Foto.Widgets.Icon(100,100);
            icon.set_halign(Align.CENTER);
            icon.set_have_close_button(false);
            icon.set_have_edit_button(false);
            rating = new Rating (album.rating);
            rating.rating_changed.connect((rating) =>{
                album.rating = (int)rating;
                album_manager.update_album(album.album_name, album);
            });

            album_manager.album_updated.connect(update_thumb);

            rating.set_halign(Align.CENTER);
            icon_box.pack_start (icon,false,false, 0);
            icon_box.pack_start (rating, false, true, 0);

            //info box left
            var info_box_left = new Gtk.Box (Orientation.VERTICAL, 0);
            album_name_label = new LLabel.markup (" ");
            info_box_left.width_request = 200;
		    description_view = new WrapLabel (null);
            description_view.use_markup = true;

            info_box_left.pack_start (album_name_label,false, true, 5);
            info_box_left.pack_start (description_view,false, true, 5);

            //info box right
            //var info_box_right = new Gtk.Box(Orientation.VERTICAL, 0);

            //style
            var style_provider = Utils.Css.get_css_provider ();
            this.get_style_context ().add_provider (style_provider, Gtk.STYLE_PROVIDER_PRIORITY_THEME);
            this.get_style_context ().add_class ("albuminfo");

            //add all
            mainbox = new Gtk.Box (Orientation.HORIZONTAL, 0);
            mainbox.pack_start (icon_box, false, true, 5);
            mainbox.pack_start (info_box_left, false, true, 5);
            this.add(mainbox);
            this.show_all();
        }


        public void set_album (Album album) {

            this.album = album;
            //var str = album.album_cover.picture_path;
            string str = null;
            if(album.picture_list.size >0)
                str = album.picture_list.get (0).picture_path;
            album_name_label.label = "<span foreground='"+ label_color +"'><b>" + album.album_name + " - " +
                                       parse_date_label (album.creation_date) + "</b></span>";
            description_view.label = "<span foreground='"+ label_color +"'>" + album.description + "</span>";
            rating.set_rating(album.rating);
            icon.set_icon (str);
            this.queue_draw();
        }

        private void update_thumb(string album_name) {

            var new_album = album_manager.get_album_by_name(album_name);
            rating.set_rating(album.rating);

            if(album == new_album && !cover_has_changed(new_album)) 
                return;

            album = new_album;
            if(album.album_cover.picture_path != null && album.album_cover.picture_path != "")
                icon.set_icon(album.album_cover.picture_path);
            else if(album.picture_list.size > 0) 
                icon.set_icon(album.picture_list.get(0).picture_path);
            else
                icon.set_icon(null);
        }

        bool cover_has_changed (Album new_album) {
            // TODO check if the previous cover photo has been deleted
            return icon.is_empty ();
        }

        private string parse_date_label(DateTime time){
       
            string datetime = "";
            if(date_format == DateFormat.YYYYDDMM)
                    datetime = time.get_year().to_string() + "-" + time.get_day_of_month().to_string() + "-" + time.get_month().to_string();
            else if(date_format == DateFormat.YYYYMMDD)
                    datetime = time.get_year().to_string() + "-" + time.get_month().to_string() + "-" + time.get_day_of_month().to_string();
            else if(date_format == DateFormat.YYYYDDMMHHMMSS)
                    datetime = time.get_year().to_string() + "-" + time.get_day_of_month().to_string() + "-" + time.get_month().to_string();
            else if(date_format == DateFormat.YYYYMMDDHHMMSS)
                    datetime = time.get_year().to_string() + "-" + time.get_month().to_string() + "-" + time.get_day_of_month().to_string();
            return datetime;
        }
    }
}
