//
//  Copyright (C) 2012-2013 Erasmo Marín
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
using Gdk;
using Gtk;
using Cairo;
using Granite.Drawing;
using Pango;
using Foto.AlbumManager;
using Foto.Dialogs;
using Foto.Views;


namespace Foto.Widgets{

private enum AlbumState{
    EMPTY,
    SETED,
    NO_PICTURE
}

public enum DateFormat{
    YYYYDDMM=1,
    YYYYMMDD=2,
    YYYYDDMMHHMMSS=3,
    YYYYMMDDHHMMSS=4
}


public class AlbumWidget : Gtk.Box {

    Album album;
	public string album_name;
    LLabel label;
    LLabel date_label;
    AlbumIcon icon;
    DateFormat date_format = DateFormat.YYYYDDMM;
    Rating rating;
    //style
    public string label_color = "#ffffff";

    public AlbumWidget.from_album(Album album, int width /*Album widget width*/, int height/*Album widget height*/){
        GLib.Object (orientation: Gtk.Orientation.VERTICAL);
        this.album = album;
        if(album.album_cover.picture_path != null && album.album_cover.picture_path != "")
            icon = new AlbumIcon(album.album_cover.picture_path, album.picture_list.size);
        else if(album.picture_list.size > 0){
            icon = new AlbumIcon(album.picture_list.get(0).picture_path, album.picture_list.size);
        }
        else {
            icon = new AlbumIcon(null, album.picture_list.size);
        }
        album_manager.album_updated.connect(update_thumb);

        //signals and handlers
        icon.icon_clicked_event.connect(() => {
            main_window.view_container.image_grid.set_album(this.album);
            while (Gtk.events_pending()) {Gtk.main_iteration(); }
            main_window.view_container.set_view_state(ViewState.IMAGEGRID_VIEWER);
        });

        icon.close_event.connect(() => {
            //dialog
            var dialog = new Gtk.MessageDialog(main_window,Gtk.DialogFlags.MODAL,Gtk.MessageType.QUESTION, 
                                                              Gtk.ButtonsType.YES_NO, 
                                                              _("Are you sure you want to permanently delete the selected album?") + "\n\n" +
                                                              _("All the containing photos will be deleted also."));
            dialog.set_title(_("You are trying to delete an Album"));

            dialog.response.connect((response)=>{
                //IF YES
                if(response == ResponseType.YES){
                    this.remove();
                }
                dialog.destroy();
            });
            dialog.run();
        });

        icon.edit_event.connect(() => {
            new_album_dialog(album);
        });

            
        label = new LLabel.markup_center("<span foreground='"+ label_color +"'>" + album.album_name + "</span>");
        if(album.creation_date != null){
            date_label = new LLabel.markup_center("<span foreground='"+ label_color +"'>" + parse_date_label(album.creation_date) + "</span>");
            date_label.sensitive = false;
        }
        album_name = album.album_name;
        init();
    }

    private void init(){
        this.set_hexpand(true);
        this.set_vexpand(false);
        label.set_ellipsize(EllipsizeMode.END);
        this.pack_start(icon, false, false, 0);
        rating = new Rating(album.rating);
        rating.rating_changed.connect((rating) =>{
            album.rating = (int)rating;
            album_manager.update_album(album.album_name, album);
        });
        rating.set_halign (Align.CENTER);
        this.pack_start(rating, false, false,0);
        this.pack_start(label, false, false, 0);
        		    
        //this should be checked when adding more photos		    
		this.has_tooltip=true;
		this.tooltip_text = album.get_n_pictures().to_string()+ " " + _("photos in album");

        if(date_label != null)
            this.pack_end(date_label, false, false, 0);
    }

	public void update_labels(){
		label.label = "<span foreground='"+ label_color +"'>" + album.album_name + "</span>";
	}

    private new void remove(){
        this.set_visible(false);
        album_manager.delete_album(album);
        main_window.view_container.album_grid.on_album_destroy(this);
        this.destroy();
    } 


    private void new_album_dialog(Album album){
            new AlbumDialog(album);
    }

    private string parse_date_label(DateTime time){
       
        string datetime = "";
        if(date_format == DateFormat.YYYYDDMM)
                datetime = time.get_year().to_string() + "-" + time.get_day_of_month().to_string() + "-" + time.get_month().to_string();
        else if(date_format == DateFormat.YYYYMMDD)
                datetime = time.get_year().to_string() + "-" + time.get_month().to_string() + "-" + time.get_day_of_month().to_string();
        else if(date_format == DateFormat.YYYYDDMMHHMMSS)
                datetime = time.get_year().to_string() + "-" + time.get_day_of_month().to_string() + "-" + time.get_month().to_string();
        else if(date_format == DateFormat.YYYYMMDDHHMMSS)
                datetime = time.get_year().to_string() + "-" + time.get_month().to_string() + "-" + time.get_day_of_month().to_string();
        return datetime;
    }


    private void update_thumb(string album_name) {
            
        if(this.album_name != album_name) return;

        if(album.album_cover.picture_path != null && album.album_cover.picture_path != "")
            icon.set_from_file(album.album_cover.picture_path, album.picture_list.size);
        else if(album.picture_list.size > 0) {
            icon.set_from_file(album.picture_list.get(0).picture_path, album.picture_list.size);
        }
        rating.set_rating(album.rating);
    }

}
}
