//  Copyright (C) 2012 Erasmo Marín
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// It's not being used anymore

using Gdk;
using Gtk;
using Cairo;

namespace Foto.Widgets{

public enum ImageState{ /*represents the las state of the image widget*/
    BESTFIT,
    ZOOMED,
    SCALED,
    NORMAL,
    CHANGED,
    NONE
}


public enum RotationType{
    LEFT,
    RIGHT
}

public class PictureContainer : Gtk.ScrolledWindow {

    public PictureWidget viewer;

    public PictureContainer(string filename){
        viewer = new PictureWidget (filename, this);
        //this.get
        this.add_with_viewport(viewer);
        this.set_policy(PolicyType.AUTOMATIC , PolicyType.AUTOMATIC);
    }

    public override bool scroll_event (EventScroll event) {
        //warning("width = %f , height = %f",this.get_hadjustment().get_value(),this.get_vadjustment().get_value());
        return false;
    }


}

//maybe would be useful to have this as public, maybe no
public class PictureWidget :  DrawingArea{

    private Gdk.Color bg_color;
    private Pixbuf pic_buffer;
    private Pixbuf scaled_pixbuf;
    public int image_width;
    public int image_height;
    private int width;
    private int height;
    private double resized_width = -1;
    private double resized_height = -1;
    private string filename;
    public static double zoom_factor_on_scroll = 1.06;
    public static double zoom_factor_on_call = 1.15;
    private ImageState imagestate;
    private int x;
    private int y;
    public ScrolledWindow scrolledwindow; /*The scrolledwindow where the widget is*/
    public bool best_fit_mode = true;

    public PictureWidget(string filename, ScrolledWindow scrolledwindow){
            this.filename = filename;
		    this.set_vexpand(true);
            this.set_hexpand(true);
            this.imagestate = ImageState.BESTFIT;
        	Gdk.Color.parse ("000000", out bg_color);
		    try {
                pic_buffer = new Pixbuf.from_file(filename);
		    } catch (GLib.Error error) {
			    warning(error.message);
                return;
		    }
            scaled_pixbuf = pic_buffer.copy();
            image_width = pic_buffer.get_width();
            image_height = pic_buffer.get_height();
            add_events (Gdk.EventMask.BUTTON_PRESS_MASK
                      | Gdk.EventMask.BUTTON_RELEASE_MASK
                      | Gdk.EventMask.POINTER_MOTION_MASK
                      | Gdk.EventMask.SCROLL_MASK );
    }


    /*public void use_transitions(bool b){
        transitions = b;
    }*/

    public void set_image(string filename){

		try {
            pic_buffer = new Pixbuf.from_file(filename);
		} catch (GLib.Error error) {
			warning(error.message);
            return;
		}
        best_fit_mode = true;
        Widget parent = get_parent();
        if(parent == null) return;
        this.filename = filename;
        image_width = pic_buffer.get_width();
        image_height = pic_buffer.get_height();
        width = parent.get_allocated_width ();
        height = parent.get_allocated_height ();
        resized_width = -1;
        resized_height = -1;
        this.set_image_state(ImageState.CHANGED);
    }


    /* Widget is asked to draw itself */
    public override bool draw (Cairo.Context cr){
            width = get_allocated_width ();
            height = get_allocated_height ();

            //draw background
            cr.rectangle(0,0,width, height);
            cr.set_source_rgb(0,0,0);
            cr.fill_preserve ();

            //draw based on draw mode
            if(imagestate == ImageState.BESTFIT || imagestate == ImageState.CHANGED )
                draw_best_fit(cr);
            else if(imagestate == ImageState.ZOOMED)
                draw_on_scroll_up(cr);
            else if(imagestate == ImageState.SCALED)
                draw_on_scroll_down(cr);
            else if(imagestate == ImageState.NONE)
                draw_none(cr);
            else if(imagestate == ImageState.NORMAL)
                draw_normal(cr);
            if(imagestate != ImageState.CHANGED || best_fit_mode)
                update_size();
        cr.paint();
        return true;
    }

    private void draw_none(Cairo.Context cr){
       x = (width - scaled_pixbuf.get_width())/2;
       y = (height - scaled_pixbuf.get_height())/2;
       Gdk.cairo_set_source_pixbuf(cr,scaled_pixbuf, x,y);
       cr.paint();
    }

    private void draw_normal(Cairo.Context cr){
       x = 0;
       y = 0;
       Gdk.cairo_set_source_pixbuf(cr,pic_buffer, x,y);
       cr.paint();
    }


    private void draw_best_fit(Cairo.Context cr){

        width = get_parent().get_allocated_width () - 4;
        height = get_parent().get_allocated_height () - 4;


        if(image_width >= width || image_height >= height){
            scaled_pixbuf = resize_pixbuf(pic_buffer, width, height);
            x = (width - scaled_pixbuf.get_width())/2;
            y = (height - scaled_pixbuf.get_height())/2;
            Gdk.cairo_set_source_pixbuf(cr,scaled_pixbuf, x,y);
            resized_width = scaled_pixbuf.get_width();
            resized_height = scaled_pixbuf.get_height();
        }
        else{
            x = (width - pic_buffer.get_width())/2;
            y = (height - pic_buffer.get_height())/2;
            Gdk.cairo_set_source_pixbuf(cr,pic_buffer, x,y);
            resized_width = pic_buffer.get_width();
            resized_height = pic_buffer.get_height();
        }
        if(!best_fit_mode)imagestate = ImageState.NONE;
        else imagestate = ImageState.BESTFIT;
    }

    //resize a pixbuf preserving the aspect ratio of original pixbuf
    private Pixbuf resize_pixbuf(Pixbuf pixbuf, int width, int height){

        double rf1, rf2;

        rf1 = (double)width/(double)pixbuf.get_width();
        rf2 = (double)height/(double)pixbuf.get_height();

        int posibility1 = (int)(pixbuf.get_height() * rf1);
        int posibility2 = (int)(pixbuf.get_width() * rf2);

        if(posibility1 <= height){
              return pic_buffer.scale_simple(width,posibility1, InterpType.BILINEAR);
        }
        else{
              return pic_buffer.scale_simple(posibility2,height, InterpType.BILINEAR);
        }
    }


    private void draw_on_scroll_up(Cairo.Context cr){
            if(resized_width>(image_width*20) && resized_width != -1) return;

            if(resized_width != -1){
                resized_width = resized_width*zoom_factor_on_scroll;
                resized_height = resized_height*zoom_factor_on_scroll;
            }
            else{
                resized_width = image_width*zoom_factor_on_scroll;
                resized_height = image_height*zoom_factor_on_scroll;
            }

            scaled_pixbuf = pic_buffer.scale_simple((int)(resized_width), 
                                                    (int)(resized_height), 
                                                    InterpType.BILINEAR);
            x = (width - (int)resized_width)/2;
            y = (height - (int)resized_height)/2;
            Gdk.cairo_set_source_pixbuf(cr,scaled_pixbuf, x,y);
            imagestate = ImageState.NONE;//reset state to NONE
    }


    private void draw_on_scroll_down(Cairo.Context cr){
            
            if(resized_width<(image_width/20) && resized_width != -1) return;

            double scaling_factor = 1 - (zoom_factor_on_scroll - 1);

            if(resized_width != -1){
                resized_width = resized_width*scaling_factor;
                resized_height = resized_height*scaling_factor;
            }
            else{
                resized_width = image_width*scaling_factor;
                resized_height = image_height*scaling_factor;
            }
            scaled_pixbuf = pic_buffer.scale_simple((int)(resized_width), 
                                                    (int)(resized_height), 
                                                    InterpType.BILINEAR);
            x = (width - (int)resized_width)/2;
            y = (height - (int)resized_height)/2;
            Gdk.cairo_set_source_pixbuf(cr,scaled_pixbuf, x,y);
            imagestate = ImageState.NONE;//reset state to NONE

    }

    //TODO: ALL
    public void rotate(RotationType rotation){

    }


    private void update_size(){

        Widget parent = get_parent();

        if(parent == null) return;

        int allocwidth, allocheight;
        //FIXME : 10 shouldn't be a static value, but get_allocated_width() is not working
        int pwidth = parent.get_allocated_width () - 10; //- scrolledwindow.get_hscrollbar().get_allocated_height();
        int pheight = parent.get_allocated_height () - 10; //- scrolledwindow.get_vscrollbar().get_allocated_width();

        if(pwidth> resized_width) 
            allocwidth = pwidth;
        else 
            allocwidth = (int)resized_width;
        if(pheight> resized_height) 
            allocheight = pheight;
        else 
            allocheight = (int)resized_height;

        this.set_size_request(allocwidth,allocheight);
    }

    public void set_image_state(ImageState state){
        this.imagestate = state;
        this.queue_draw();
    }


    /* Mouse button got pressed over widget */
    public override bool button_press_event (Gdk.EventButton event) {
        return false;
    }

    /* Mouse button got released */
    public override bool button_release_event (Gdk.EventButton event) {
        return false;
    }

    /* Mouse pointer moved over widget */
    public override bool motion_notify_event (Gdk.EventMotion event) {
        // ...
        return false;
    }

    public override bool scroll_event (EventScroll event) {
        best_fit_mode = false;
        if(event.direction == ScrollDirection.UP){
            this.set_image_state(ImageState.ZOOMED);
        }
        else if(event.direction == ScrollDirection.DOWN){
            this.set_image_state(ImageState.SCALED);
        }
        
        return false;
    }
}
}
