/***
BEGIN LICENSE
Copyright (C) 2011 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/

using Gdk;
using Gtk;
using Utils;

namespace Foto.Widgets {

    public class FotoThumbnailPane : Gtk.ScrolledWindow{

	private FotoThumbnailPane_i pane;	
	private Gtk.TreeIter selected_photo;
	private Foto.Views.FotoViewer fotoviewer;
	public static int WIDTH = 140;

	public FotoThumbnailPane (Foto.Views.FotoViewer fotoviewer, ImageList image_list){
		this.fotoviewer = fotoviewer;
		this.set_policy (Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
		set_imagelist (image_list);
	}

	private void set_imagelist(ImageList image_list){
		pane = new FotoThumbnailPane_i (image_list);
		this.add (pane);
		this.set_min_content_width (WIDTH);
		this.show_all ();
	}

    public void load_thumbs (){

	    pane.newmodel = new Gtk.ListStore (2, typeof (Gdk.Pixbuf), typeof (string));
	    pane.set_selection_mode (Gtk.SelectionMode.SINGLE);
	    pane.set_pixbuf_column (0);
	    pane.set_model (pane.newmodel);

	    string icon_style = """
		        .thumbnail-view {
                    background-color: #ffffff;
                }
                .thumbnail-view:selected {
                    background-color: #5a9adc;
                    border-color: shade (mix (rgb (34, 255, 120), #fff, 0.5), 0.9);
                }
            """;

	    var icon_view_style = new Gtk.CssProvider ();

            try {
                icon_view_style.load_from_data (icon_style, -1);
            } catch (Error e) {
                warning (e.message);
            }

        pane.get_style_context ().add_class ("thumbnail-view");
	    pane.get_style_context ().add_provider (icon_view_style, Gtk.STYLE_PROVIDER_PRIORITY_THEME);
	    //Add thumbnails to the iconview
	    string buff1, buff2;
	    for(int i=1; i<pane.image_list.size; i++){
		buff1 = pane.image_list.get_full_filename (i);
		buff2 = pane.image_list.get_filename (i);
		load_image_async.begin (buff2, buff1, (obj, async_res) => {
     		 GLib.debug ("Finished loading.");
  		  });
	    }
	    pane.selection_changed.connect (update_selected);
	    this.queue_draw ();
    }

    private async void load_image_async (string filename, string full_filename) {
	  GLib.File file = GLib.File.new_for_commandline_arg (full_filename);
	  try {
	    GLib.InputStream stream = yield file.read_async ();
	    Pixbuf image = yield new Pixbuf.from_stream_at_scale_async (stream, 110, -1, true);
   	    // Add the image name and thumbnail to the IconView
	    Gtk.TreeIter root;
	    pane.newmodel.append (out root);
	    pane.newmodel.set (root, 0, image, -1);
	    pane.newmodel.set (root, 1, filename, -1);
        pane.iters.append (root);	    
	    debug ("Added %s to thumbnail\n", filename);
	  } catch ( GLib.Error e ) {
	    GLib.error (e.message);
	  }
    }


    private void update_selected () {
		    var selected = pane.get_selected_items ();
            if (selected.length() == 1) {
                GLib.Value filename;
                // Get the filename of the selected image.
                var item = selected.nth_data(0);
                pane.newmodel.get_iter (out this.selected_photo, item);
                pane.newmodel.get_value (this.selected_photo, 1, out filename);
                debug ("Selected: %s", filename.get_string ());
		        fotoviewer.draw_from_filename.begin (filename.get_string ());
            }
    }

   }

//Based on wallpaper-plug
    private class FotoThumbnailPane_i : Gtk.IconView{
        public Gtk.ListStore newmodel;
	    public GLib.List <Gtk.TreeIter?> iters = new GLib.List <Gtk.TreeIter?> ();
	    public ImageList image_list;

        public FotoThumbnailPane_i (ImageList image_list) {
		this.image_list = image_list;
       }
    }
 }
