/***
BEGIN LICENSE
Copyright (C) 2011 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/

using Gtk;
using Gdk;
using Granite.Widgets;
using Foto.Views;
using Foto.Dialogs;

namespace Foto.Widgets {

    public class FotoToolbar : Gtk.Toolbar{
     
        // Chrome widgets
        Gtk.Entry search_box;
        Gtk.Button nav_button_prev;
        Gtk.Button nav_button_next;
        Gtk.Button slideshow_button;
        Gtk.Button restore_button;
        Gtk.Button home_button;
        Gtk.Button imagegrid_button;
        //Gtk.Button imageviewer_button;
        ButtonGroup nav_button;
        ButtonGroup view_button;
        ButtonGroup image_options;
        Granite.Widgets.ToolButtonWithMenu export;
        //Deprecated TODO: Replacement
        //Granite.Widgets.ContractorMenu export_menu;
        //Gtk.Scale scale;
        Gtk.ToolItem lspace = new Gtk.ToolItem ();
        Gtk.ToolItem rspace = new Gtk.ToolItem ();
	    private MainWindow window;

        public FotoToolbar(MainWindow window, Granite.Widgets.AppMenu app_menu) {

	        this.window = window;
	        app_menu.vertical_menu_position = ToolButtonWithMenu.VMenuPosition.TOP;
            // Global toolbar widgets
            
            this.get_style_context ().add_class ("bottom-toolbar");

			//Scale widget
			
			/*scale = new Gtk.Scale.with_range (Orientation.HORIZONTAL,0.5, 2, 0.01);
 			scale.set_draw_value (false);
 			scale.set_size_request (160, scale.get_allocated_height());
 			
 			var zoom_in_image = new Image.from_file (Build.PKGDATADIR + "/zoom-in.png");
 			var zoom_out_image = new Image.from_file (Build.PKGDATADIR + "/zoom-out.png");
 			
 			var scale_container = new ToolItem ();
            scale_container.add(scale);
            
 			var zc1 = new ToolItem(); 
            zc1.add(zoom_out_image);
            
           	var zc2 = new ToolItem();
           	zc2.add(zoom_in_image);*/

            // Spacing
            this.draw.connect((cr) => {
            	center_nav_button();
            	return false;
            });
            	
            rspace.set_expand(true);

            // Searchbar
            search_box = new Gtk.Entry ();
            search_box.placeholder_text = _("Search");
            search_box.primary_icon_stock = "gtk-find";
            
            var find_toolitem = new Gtk.ToolItem ();
            find_toolitem.add(search_box);
            
           	search_box.changed.connect(on_search);

            // Nav button
            nav_button = new ButtonGroup();
            var nav_button_container = new ToolItem();        
            nav_button_container.add(nav_button);
            
            nav_button_prev = nav_button.append_button_with_icon ("go-previous-symbolic", Gtk.IconSize.SMALL_TOOLBAR);
            nav_button_prev.set_tooltip_text (_("Show previous picture"));
            nav_button_prev.clicked.connect (draw_prev);
            
            
            slideshow_button = nav_button.append_button_with_icon ("view-fullscreen-symbolic", Gtk.IconSize.SMALL_TOOLBAR);
            slideshow_button.set_tooltip_text (_("Start presentation"));
            slideshow_button.clicked.connect (on_fullscreen);
            
            restore_button = nav_button.append_button_with_icon ("view-restore-symbolic", Gtk.IconSize.SMALL_TOOLBAR);
            restore_button.clicked.connect (on_restore);           
            
            window.fullscreen_toggled.connect(update_state);
            
            nav_button_next = nav_button.append_button_with_icon ("go-next-symbolic", Gtk.IconSize.SMALL_TOOLBAR);
            nav_button_next.set_tooltip_text (_("Show next picture"));
            nav_button_next.clicked.connect (draw_next);
            
            
            // View button
            view_button = new ButtonGroup();
            var view_button_container = new ToolItem();
            view_button_container.add(view_button);
            
            home_button = view_button.append_button_with_icon ("go-home-symbolic", Gtk.IconSize.SMALL_TOOLBAR);
            home_button.set_tooltip_text (_("Go to album view"));
            home_button.clicked.connect(go_home);
            
            imagegrid_button = view_button.append_button_with_icon ("view-grid-symbolic", Gtk.IconSize.SMALL_TOOLBAR);
            imagegrid_button.set_tooltip_text (_("Comeback to the album"));    
            imagegrid_button.clicked.connect(go_to_album);
            //contractor button
            /*export_menu = new Granite.Widgets.ContractorMenu("","image");
            //this is ugly, ToolButtonWithMenu should have a "from_icon" constructor
            export = new ToolButtonWithMenu (new Gtk.Image.from_icon_name ("document-export-symbolic", Gtk.IconSize.SMALL_TOOLBAR), 
                                              _("Export or Share this image"), 
                                              export_menu);
            export.clicked.connect(on_export);
	        export.vertical_menu_position = ToolButtonWithMenu.VMenuPosition.TOP;

            image_options = new ButtonGroup();
            image_options.add(export);

            var image_options_container = new ToolItem();
            image_options_container.add(image_options);*/
            
            //add to toolbar
            this.insert (view_button_container, 0);
            this.insert (lspace, 1);
           	this.insert (nav_button_container,2);
            this.insert (rspace, 3);
            /*this.insert (image_options_container, 4);*/
            this.insert (find_toolitem, 4);
            this.insert (app_menu, 5);
            this.show_all ();
            
            window.view_container.view_state_changed.connect (update_state);
            window.key_press_event.connect (on_key_press);
       }

        /*public void on_export () {
            if(export.get_active()){
                string filename = window.view_container.fotoviewer.get_file_path();
                export_menu.update(filename, "image");
            }
        }*/
       
        public void on_fullscreen () {
			window.fullscreen ();
        }
       
        public void on_restore () {
		    window.unfullscreen();
        }       
       
        public void center_nav_button () {
			int width = get_center() - nav_button.get_allocated_width()/2 - view_button.get_allocated_width();
			lspace.set_size_request (width, lspace.get_allocated_height());
        }
       
       public int get_center (){
       		return this.get_allocated_width()/2;
       }

		public void on_search (){
			window.view_container.search(search_box.get_text());					
		}

	    public void draw_next (){
		    window.fotoviewer.draw_next.begin();
	    }

	    public void draw_prev (){
		    window.fotoviewer.draw_prev.begin();
	    }

        public void update_state (){
        
            if(window.view_container.get_view_state() == ViewState.ALBUM_VIEWER){
				view_button.set_visible(false);
				nav_button.set_visible(false);
                image_options.set_visible(false);
                search_box.set_visible(true);       
            }
            
            else if(window.view_container.get_view_state() == ViewState.IMAGEGRID_VIEWER){
            	view_button.set_visible(true);
            	home_button.set_visible(true);
            	imagegrid_button.set_visible(false);
            	nav_button.set_visible(false);
                image_options.set_visible(false);
                search_box.set_visible(true);       
            }
            
            else{
            	view_button.set_visible(true);
            	home_button.set_visible(true);
            	imagegrid_button.set_visible(true);
           		nav_button.set_visible(true);
                image_options.set_visible(true);
                search_box.set_visible(false);       
            }
            
            if(window.is_fullscreen){
            	restore_button.set_visible(true);
            	slideshow_button.set_visible(false);
            } else {
            	restore_button.set_visible(false);
            	slideshow_button.set_visible(true);            
            }  
        }

        public bool on_key_press(EventKey e){

            if(window.view_container.get_view_state() == ViewState.IMAGE_VIEWER){

                switch (e.keyval) {
                    case Gdk.Key.Left:
                        nav_button_prev.clicked ();
                        break;
                    case Gdk.Key.Right:
                        nav_button_next.clicked ();
                        break;
                }
            }

            return false;
        }


        public void go_home(){
			window.view_container.set_view_state(ViewState.ALBUM_VIEWER);
        }

        public void go_to_album(){
			window.view_container.set_view_state(ViewState.IMAGEGRID_VIEWER);
        }

    }
 }
