//
//  Copyright (C) 2012 Erasmo Marín
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
using Gdk;
using Gtk;
using Cairo;
using Granite.Drawing;
using Pango;
using Foto.AlbumManager;
using Foto.Dialogs;
using Foto.Views;
using Utils;


namespace Foto.Widgets{

//the state of the icon
private enum IconState {
    EMPTY,
    SETED
}

//this icon is used in album grid view and image grid view
private class Icon :  DrawingArea{

	//the icon pixbuf
    private Pixbuf cover;
    public string cover_filename;
    private Pixbuf close_button;
    private Pixbuf edit_button;
    private StyleContext style_context;
    
    //the state of the icon, EMPTY or SETED
    private IconState state;
    
    //some values
    private int center_value=20;
    private int margin_x=10;
    private int margin_y=10;
    private int width=0;
    private int height=0;
    
    //close button values
    private int close_x;
    private int close_y;
    
    //edit button values
    private int edit_x;
    private int edit_y;
    
    //style
    public RGBA dash_color;
    public RGBA dash_color_prelight;
    public RGBA icon_line_color;
    
    //booleans
    private bool show_prelight_frame = true;
    private bool show_close_button = true;
    private bool show_edit_button = true;
    private bool draw_icon_shadow = true;
    
    //signals   
    public virtual signal void edit_event(){
        debug("Edit event");
    }
    public virtual signal void close_event(){
        debug("Close event");
    }
    public virtual signal void icon_clicked_event(){
        debug("Icon clicked event");
    }

    public Icon.from_image_file(string? file, int width /*icon width*/, int height/*icon height*/){
            cover_filename = file;
            this.width = width;
            this.height = height;
            set_size_request(width,height);
            state = IconState.SETED;
            cover = Utils.IconFactory.get_icon(Utils.IconFactory.IconType.ICON_NO_PICTURE);
            load_image_async (file);

            if(state == IconState.SETED){

                margin_x = (this.get_allocated_width() - cover.get_width())/2;
                margin_y = (this. get_allocated_height() - cover.get_height())/2;
            }
            init_ui();
    }
    //creates an icon from a pixbuf
    public Icon.from_pixbuf(Gdk.Pixbuf? pixbuf_icon){
    
    		if(pixbuf_icon != null){
                var shadow_color = new Granite.Drawing.Color(0,0,0,255);
    			cover = PixbufUtils.render_drop_shadow (pixbuf_icon, 3, 190, shadow_color);    
            	set_size_request(width,height);
            	this.width = cover.get_width();
            	this.height = cover.get_height();
            	state = IconState.SETED;
            }
            else{
                state = IconState.EMPTY;
            }      
            init_ui();
    }

	//creates an empty icon
    public Icon(int width, int height){
            set_size_request(width,height);
            this.width = width;
            this.height = height;
            state = IconState.EMPTY;        
            init_ui();
    }
   
    private void init_ui(){

			close_button = Utils.IconFactory.get_icon(Utils.IconFactory.IconType.ICON_CLOSE_BUTTON);
			edit_button = Utils.IconFactory.get_icon(Utils.IconFactory.IconType.ICON_EDIT_BUTTON);

            //default style
            style_context = new StyleContext();
            var widgetpath = new WidgetPath();
            widgetpath.append_for_widget(this);
            style_context.set_path(widgetpath);
            style_context.set_screen(this.get_screen());
            var style_provider = Utils.Css.get_css_provider ();
            style_context.add_provider (style_provider, 600);
            style_context.add_class ("icon");
            dash_color = style_context.get_border_color (StateFlags.NORMAL);/*{1.0, 1.0, 1.0, 0.55};*/
            dash_color_prelight = style_context.get_border_color (StateFlags.PRELIGHT);/*{1.0, 1.0, 1.0, 0.70};*/
            icon_line_color = style_context.get_color (StateFlags.NORMAL); /*{1.0, 1.0, 1.0, 0.95};*/

            add_events (Gdk.EventMask.BUTTON_PRESS_MASK
                        | Gdk.EventMask.BUTTON_RELEASE_MASK 
                        | Gdk.EventMask.LEAVE_NOTIFY_MASK 
                        | Gdk.EventMask.ENTER_NOTIFY_MASK );

            set_vexpand(false);
            set_hexpand(false);
            button_release_event.connect((event) => { 

                //if user clicks close button
                if(show_close_button && event.x>= close_x && event.x<= close_x+32 && event.y>= close_y && event.y <= close_y +32){
			        close_event();
                    this.set_state_flags(StateFlags.NORMAL,true);
                return false;
                }

                //if user clicks edit button
                else if(show_edit_button && event.x>= edit_x && event.x<= edit_x+32 && event.y>= edit_y && event.y <= edit_y +32){
			        edit_event();
                    this.set_state_flags(StateFlags.NORMAL,true);
                    return false;
                }

                //if user clicks icon 
                else{
			        icon_clicked_event();
                    this.set_state_flags(StateFlags.NORMAL,true);
                    return false;
                }     
            });
    }

	//SETERS

	//prelight frame is a rounded square drawn when the icon is in prelight state
	public void set_have_prelight_frame(bool frame){
		show_prelight_frame = frame;
	}

	//close icon is an small button that is drawn on prelight in left corner of the widget, and allow to the user to destroy the icon
	public void set_have_close_button(bool close_button){
		show_close_button = close_button;
	}
	
	//edit icon is an small button that is drawn on prelight in right corner of widget, and allow to the user to do what ever the programmer wants
	public void set_have_edit_button(bool edit_button){
		show_edit_button = edit_button;
	}

	//Showing a black shadow is optional
	public void set_have_icon_shadow(bool shadow){
		draw_icon_shadow = shadow;
	}
	
    //DRAW
    public override bool draw (Cairo.Context cr){

        if(state == IconState.EMPTY){

            margin_x = (this.get_allocated_width() - width)/2;
            margin_y = (this. get_allocated_height() - height)/2;
            //draw rectangle
            if(state == IconState.EMPTY && get_state_flags() == StateFlags.PRELIGHT)
                cr.set_source_rgba (dash_color_prelight.red, 
                                    dash_color_prelight.green, 
                                    dash_color_prelight.blue, 
                                    1);
            else
                cr.set_source_rgba (dash_color.red, 
                                    dash_color.green, 
                                    dash_color.blue, 
                                    0.7);

            Utilities.cairo_rounded_rectangle (cr, margin_x + center_value/2, margin_y + center_value/2,width - center_value,height - center_value, 5);

            cr.set_dash (new double[] {(137.999)/19,(137.95)/15.905},6);
            cr.set_line_width(4);
            cr.stroke ();
            cr.set_dash (null, 0);
        }
        else{
            if(get_state_flags() == StateFlags.PRELIGHT && show_prelight_frame){
                cr.set_source_rgba (icon_line_color.red, 
                                    icon_line_color.green, 
                                    icon_line_color.blue, 
                                    0.95);

                Utilities.cairo_rounded_rectangle (cr, margin_x,margin_y,cover.get_width(),cover.get_height(), 5);
                cr.fill ();
                cr.set_line_width(1);
                cr.set_source_rgba (1, 1, 1, 0.2);
                Utilities.cairo_rounded_rectangle (cr, margin_x,margin_y,cover.get_width(),cover.get_height(), 5);
                cr.stroke();
            }

            margin_x = (this.get_allocated_width() - cover.get_width())/2;
            margin_y = (this. get_allocated_height() - cover.get_height())/2;
            Gdk.cairo_set_source_pixbuf(cr,cover, margin_x, margin_y);
            cr.paint();
            }

        //We show close and edit buttons even if icon is empty
        //draw close button
        if(show_close_button && close_button != null && get_state_flags() == StateFlags.PRELIGHT){
        	close_x = margin_x-3;
        	close_y = margin_y-3;
        	Gdk.cairo_set_source_pixbuf(cr,close_button,close_x,close_y);
        	cr.paint();
        }        
        
        //draw edit button
        if(show_edit_button && edit_button != null && get_state_flags() == StateFlags.PRELIGHT){
        	edit_x = this.get_allocated_width() - 32 - margin_x;
        	edit_y = margin_y-3;
        	Gdk.cairo_set_source_pixbuf(cr,edit_button, edit_x, edit_y);
        	cr.paint();
        }        
        
        return false;
    }

    public void set_icon (string? filename) {

        load_image_async (filename);

    }

    private async void load_image_async (string? filename) {

        if (filename == null) {
            empty_icon();
    		return;
        }

        set_have_prelight_frame(false);
        GLib.File file = GLib.File.new_for_commandline_arg (filename);

    	if (!file.query_exists ()) {
            empty_icon();
    		return;
        }

	    try {
            Pixbuf image = null;
            if(settings.use_cache) {
                image = yield Cache.get_cached_image_async (filename);
            }
            if(image == null){
	            GLib.InputStream stream = yield file.read_async ();
	            image = yield new Pixbuf.from_stream_at_scale_async (stream,width-margin,height-margin,true);
                Cache.cache_image(filename, 150, 150);
            } else if (image.width > (width-margin) || image.height > (height-margin)) {
	            GLib.InputStream stream = yield file.read_async ();
	            image = yield new Pixbuf.from_stream_at_scale_async (stream,width-margin,height-margin,true);
            }
            if(draw_icon_shadow) {
                var shadow_color = new Granite.Drawing.Color(0,0,0,255);
                image = PixbufUtils.render_drop_shadow (image, 3, 190, shadow_color);
            }
            this.cover = image;
            set_have_prelight_frame (true);
            state = IconState.SETED;
            this.queue_draw();
	    } catch ( GLib.Error e ) {
            empty_icon();
	        GLib.error (e.message);
	    }
    }

    private void empty_icon() {
        state = IconState.EMPTY;
        this.queue_draw();
    }

    public bool is_empty () {
        return state == IconState.EMPTY;
    }

    /*public new StyleContext get_style_context() {
        return style_context;
    }*/

	//EVENTS

    /* Mouse button got pressed over widget */
    public override bool button_press_event (Gdk.EventButton event) {
        this.set_state_flags(StateFlags.FOCUSED,true);
        return false;
    }

    /* Mouse button got released */
    public override bool button_release_event (Gdk.EventButton event) {
        return false;
    }

    public override bool enter_notify_event (Gdk.EventCrossing event) {
        this.set_state_flags(StateFlags.PRELIGHT,true);
        return false;
    }

    public override bool leave_notify_event (EventCrossing event){
        this.set_state_flags(StateFlags.NORMAL,true);
        return false;
    }
}
}
