/***
BEGIN LICENSE
Copyright (C) 2011 Erasmo Marín <erasmo.marin@gmail.com>
This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 3, as published
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranties of
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
END LICENSE
***/
using Gee;
using Gtk;
using Gdk;
using Granite.Widgets;
using Foto.Views;
using Foto.AlbumManager;
using Utils;


namespace Foto.Widgets {

    public class MainWindow : Gtk.Window {    

        // Chrome widgets
	    public FotoToolbar toolbar;
	    public FotoViewContainer view_container;
	    public bool is_fullscreen = false;
	    public signal void fullscreen_toggled (bool is_fullscreen);
        // Content area widgets
        private Gtk.Box vbox;
        public Gtk.Box toolbar_container;
        public Gtk.Window fullscreen_toolbar_container;
	    private bool foto_view;
	    //gtk menu
	    public FotoMenu menu;
        //settings
        private Settings.SavedState saved_state;
        //convenient reference to views
	    public FotoAlbumGrid album_grid;
	    public FotoViewer fotoviewer;

        //pointer stuff
        private double x_pointer_position = 0;
        private double y_pointer_position = 0;
        private double last_x_pointer_position = 0;
        private double last_y_pointer_position = 0;

        public MainWindow (Granite.Widgets.AppMenu app_menu, FotoMenu menu, string[] _args) {

            set_application (fotoapp);

            // Set up defaults
            this.title = "Foto";
	        this.foto_view = false;
	        this.menu = menu;
	        
            // Set up window
            saved_state = new Settings.SavedState ();
            this.default_height = saved_state.window_height;
            this.default_width = saved_state.window_width;
            this.window_position = Gtk.WindowPosition.CENTER;
            this.delete_event.connect (on_window_delete_event);
            this.destroy.connect ( () => shutdown() );
	        this.set_border_width (0);

	        //Creating the view container with the views inside it
	        view_container = new FotoViewContainer (_args, this);
            album_grid = view_container.album_grid;
            fotoviewer = view_container.fotoviewer;
            view_container.get_view_state ();

            //toolbar
            toolbar = new FotoToolbar (this, app_menu);
            toolbar_container = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            toolbar_container.add (toolbar);

            //window as TOPLEVEL because we need to have the keyboard focus, but still it should work like a popup
            fullscreen_toolbar_container = new Gtk.Window (Gtk.WindowType.TOPLEVEL);
            fullscreen_toolbar_container.set_decorated (false);
            fullscreen_toolbar_container.set_type_hint (Gdk.WindowTypeHint.POPUP_MENU);
            fullscreen_toolbar_container.can_focus = true;
            fullscreen_toolbar_container.set_transient_for (this); //with this, we prevent to show panel
            this.fullscreen_toggled.connect (toolbar_on_fullscreen);
            //window
            vbox = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            vbox.set_border_width (0);
            vbox.pack_start (view_container, false, true, 0);
            vbox.pack_start (toolbar_container, false, false,0);

            this.key_press_event.connect (on_fullscreen_keys);
            fullscreen_toolbar_container.key_press_event.connect (on_fullscreen_keys);
            this.motion_notify_event.connect( (e) => on_pointer_motion (e) );
            fullscreen_toolbar_container.motion_notify_event.connect( (e) => update_cursor_position (e) );
            //with this we hide cursor when it stop moving
            GLib.Timeout.add(500,on_pointer_motion_stop);


            this.add (vbox);
            this.show_all ();
            toolbar.update_state ();
       }
       
        public new void fullscreen () {
     	    base.fullscreen ();
     	    is_fullscreen = true;
		    fullscreen_toggled (is_fullscreen);
        }
     
        public new void unfullscreen () {
     	    base.unfullscreen ();
     	    is_fullscreen = false;
		    fullscreen_toggled (is_fullscreen);
        }

        public void toggle_fullscreen () {
            if (is_fullscreen)
                unfullscreen ();
            else
                fullscreen ();
        }
     
        private void shutdown () {
                Gtk.main_quit ();
        }

        private bool on_fullscreen_keys (EventKey e) {
            switch (e.keyval) {
                case Gdk.Key.Escape:
                    if (is_fullscreen) 
                        unfullscreen ();
                    break;
                case Gdk.Key.F11:
                    toggle_fullscreen ();
                    break;
                /*case Gdk.Key.Super_L:
                case Gdk.Key.Super_R:
                    shortcut_popup.set_visible(!shortcut_popup.get_visible());
                    shortcut_popup.show_all();
                    break;*/
            }
            return false;
        }

        private bool on_window_delete_event (Gdk.EventAny event) {
            update_saved_state ();
            return false;
        }
     
        private void toolbar_on_fullscreen () {
            if (is_fullscreen) {
                toolbar_container.remove (toolbar);
                fullscreen_toolbar_container.add (toolbar);
                fullscreen_toolbar_container.resize (this.get_screen ().get_width (), this.toolbar.get_allocated_height () );
                fullscreen_toolbar_container.set_keep_above (true);
                fullscreen_toolbar_container.move (0,this.get_screen ().get_height () - this.toolbar.get_allocated_height () );
                hide_cursor();
            } else {
                GLib.Timeout.add(500,show_cursor);
                fullscreen_toolbar_container.remove (toolbar);
                toolbar_container.add (toolbar);
                fullscreen_toolbar_container.set_visible (false);
            }
        }

        private bool on_pointer_motion (EventMotion e) {
            if (is_fullscreen) {
                show_cursor();
                update_cursor_position(e);
                if (e.y_root >= this.get_screen ().get_height () - 10) {
                    fullscreen_toolbar_container.move (0,this.get_screen ().get_height () - this.toolbar.get_allocated_height () );
                    fullscreen_toolbar_container.set_visible (true);
                } else {
                    fullscreen_toolbar_container.set_visible (false);
                }
            }
            return false;
        }

        private bool update_cursor_position (EventMotion e) {
            x_pointer_position = e.x;
            y_pointer_position = e.y;
            return false;
        }

        private bool on_pointer_motion_stop(){
            if (is_fullscreen) {
                //if cursor hasn't moved
                if(last_x_pointer_position == x_pointer_position && last_y_pointer_position == y_pointer_position){
                    hide_cursor();
                } else {
                    last_x_pointer_position = x_pointer_position;
                    last_y_pointer_position = y_pointer_position;
                }
            }
            return true;
        }

        private void set_cursor(Gdk.CursorType type){
            var last = new Gdk.Cursor(type);
            Gdk.Window window;
            window = this.get_root_window();
            window.set_cursor(last);
        }

        private bool show_cursor(){
            this.set_cursor(Gdk.CursorType.ARROW);
            return false;
        }

        private bool hide_cursor(){
            this.set_cursor(Gdk.CursorType.BLANK_CURSOR);
            return false;
        }

        private void update_saved_state () {
            debug("Updating saved state");
            // Save window state
            if ((this.get_window ().get_state () & Settings.WindowState.MAXIMIZED) != 0)
                saved_state.window_state = Settings.WindowState.MAXIMIZED;
            else if ((this.get_window ().get_state () & Settings.WindowState.FULLSCREEN) != 0)
                saved_state.window_state = Settings.WindowState.FULLSCREEN;
            else
                saved_state.window_state = Settings.WindowState.NORMAL;

            // Save window size
            if (saved_state.window_state == Settings.WindowState.NORMAL) {
                int width, height;
                this.get_size (out width, out height);
                saved_state.window_width = width;
                saved_state.window_height = height;
            }
        }
    }
 }
