using Clutter;
using Gtk;
using GtkClutter;

namespace Foto.Widgets{

public class PictureContainer : GtkClutter.Embed {

    public PictureWidget viewer;

    public PictureContainer(string? filename){
        viewer = new PictureWidget(filename, (Clutter.Stage)this.get_stage());
        viewer.stage.scroll_event.connect((ScrollEvent) => { 
            viewer.zoom_animation(ScrollEvent);
            return false; 
        });
        this.draw.connect (on_resize);
        //this.show_all();
    }

    public bool on_resize(){
        viewer.draw_best_fit();
        return false;
    }

}


public class PictureWidget{

    public Clutter.Actor stage;
    Clutter.Texture image;
    Timeline timeline;
    double zoom = 0.5;
    double scroll_steps = 3;
    double SCROLL_STEPS = 3;
    double last_scroll=0;
    double diff_scrolls;
    double zoom_scroll_factor;
    double last_width;
    double last_height;
    bool best_fit = true;
    public bool is_at_center = true;
    public signal void image_changed(string filepath);

    public PictureWidget(string? filename, Clutter.Stage stage){
    
        this.stage = stage;
        //background color
		set_background_color();
		settings.changed.connect(set_background_color);
		

        if(filename != null) {
		    try {
                image = new Clutter.Texture.from_file(filename);
		    } catch (GLib.Error error) {
			    warning(error.message);
		    }
        } else {
            image = new Clutter.Texture();    
        }

        //calcule and set the zoom to fit image on the stage
        compute_bestfit_zoom();
        image.set_scale(zoom,zoom);
        last_width = image.get_width() * zoom;
        last_height = image.get_height() * zoom;
        image.add_action ( new DragAction() );
        image.set_reactive (true);
        stage.add_actor(image);
        center_position();
    }

    public void set_image(string filepath){
		try {
            image.set_from_file(filepath);
		} catch (GLib.Error error) {
			warning(error.message);
            return;
		}
        compute_bestfit_zoom();
        image.set_scale(zoom,zoom);
        last_width = image.get_width() * zoom;
        last_height = image.get_height() * zoom;
        center_position();
    }
    
    public void set_background_color(){
    	Gdk.Color color;
    	Gdk.Color.parse(settings.image_viewer_background_color, out color);
    	Clutter.Color stage_color = { color.red/255, color.green/255, color.blue/255, 255 };
        stage.set_background_color(stage_color);
    }

    public void draw_best_fit(){
        if(!best_fit) return;
        compute_bestfit_zoom();
        if(zoom>1) zoom = 1;
        image.set_scale(zoom,zoom);
        last_width = image.get_width() * zoom;
        last_height = image.get_height() * zoom;
        center_position();
    }

    private void compute_bestfit_zoom(){

        float rf1, rf2;

        rf1 = stage.get_width()/image.get_width();
        rf2 = stage.get_height()/image.get_height();

        float posibility = image.get_height() * rf1;

        if(posibility <= stage.get_height())
              zoom = rf1;
        else
              zoom = rf2;
    }


    private void center_position(){
         double x_margin=(stage.get_width() - image.get_width()*zoom)/2;
         double y_margin=(stage.get_height() - image.get_height()*zoom)/2;
         image.set_position((float)x_margin, (float)y_margin);
    }

    private void compute_position_on_zoom_out(){
         double x_pos = image.get_x() - (image.get_width()*zoom - last_width)/2;
         double y_pos = image.get_y() - (image.get_height()*zoom - last_height)/2;
         image.set_position((float)x_pos, (float)y_pos);
    }

    private void compute_position_on_zoom_in(){
         double x_pos = image.get_x() + (last_width - image.get_width()*zoom)/2;
         double y_pos = image.get_y() + (last_height - image.get_height()*zoom)/2;
         image.set_position((float)x_pos, (float)y_pos);
    }


    public void zoom_animation(ScrollEvent event) {
        best_fit = false;
        timeline = new Clutter.Timeline(60);
        //we add more steps if user scrolls more, it will get a more fluid zoom
        //FIXME: We should increment speed also
        scroll_steps+=1;
        diff_scrolls = event.time-last_scroll;
        last_scroll = event.time;
        zoom_scroll_factor = diff_scrolls/1000000;
        if(event.direction == ScrollDirection.DOWN )
            timeline.new_frame.connect (() => { zoom_in(); });
        else if(event.direction == ScrollDirection.UP)
            timeline.new_frame.connect (() => { zoom_out(); });
        else return;
        timeline.set_loop(true);
        timeline.start();
    }

    public void zoom_in(){
        last_width = image.get_width() * zoom;
        last_height = image.get_height() * zoom;

        zoom-=0.01;
        scroll_steps--;
       if(scroll_steps<=0 || zoom<=0) {
            if(zoom<0)zoom = 0;
            timeline.stop();
            scroll_steps=SCROLL_STEPS;
            return;
        }
        image.set_scale(zoom,zoom);
        compute_position_on_zoom_in();
    }

    public void zoom_out(){
        last_width = image.get_width() * zoom;
        last_height = image.get_height() * zoom;
        zoom+=0.01;
        scroll_steps--;
       if(scroll_steps<=0 || zoom>=5) {
            timeline.stop();
            scroll_steps=SCROLL_STEPS;
            return;
        }
        image.set_scale(zoom,zoom);
        compute_position_on_zoom_out();
    }
}
}
