//
//  Copyright (C) 2012 Erasmo Marín
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using Gtk;
using Gdk;

namespace Foto.Widgets{

public class Rating : Gtk.DrawingArea{

    private Pixbuf active_star;
    private Pixbuf inactive_star;
    private int star_width = 0;
    private int star_height = 0;
    private uint rating = 0;
    public signal void rating_changed(uint rating);


    public Rating (uint rating){
        if(rating > 5) {
            warning("Rating value may not be greater than 5, rating value will be setted to 5");
            rating = 5;
        }

        active_star = Utils.IconFactory.get_icon(Utils.IconFactory.IconType.STAR_ACTIVE_SYMBOLIC);
        inactive_star = Utils.IconFactory.get_icon(Utils.IconFactory.IconType.STAR_INACTIVE_SYMBOLIC);

        this.rating = rating;
        this.star_width = active_star.get_width();
        this.star_height = active_star.get_height();

        this.height_request = star_height;
        this.width_request = star_width * 5;

        this.set_hexpand(true);
        this.set_vexpand(true);

        add_events (  Gdk.EventMask.BUTTON_PRESS_MASK
                    | Gdk.EventMask.BUTTON_RELEASE_MASK);

        button_release_event.connect(on_star_clicked);

    }

    public void set_rating (uint rating) {
        if(rating <=5){
            this.rating = rating;
            this.queue_draw();
        } else {
            warning("Rating value may not be greater than 5, rating value will be setted to 5");
            rating = 5;
        }
    }

    private bool on_star_clicked (EventButton event) {
        
        uint new_rating = (uint)(event.x/16) + 1;
        uint old_rating = rating;

        if(new_rating == 1) { //if rating is 1, we toggle the star
            if(rating == 1)
                rating = 0;
            else
                rating = 1;
        }
        else {
            rating = new_rating;
        }

        if(old_rating != rating){ //draw and emmit signal only if value changed
            queue_draw ();
            rating_changed(rating); 
        }
        return false;
    }    


    //DRAW
    public override bool draw (Cairo.Context cr){

        uint i;
        int margin_x = 0;

        
        for(i=0; i<rating; i++) {
            Gdk.cairo_set_source_pixbuf(cr,active_star, margin_x, 0);
            cr.paint();
            margin_x+=star_width;
        }

        for(i=rating; i<5; i++) {
            Gdk.cairo_set_source_pixbuf(cr,inactive_star, margin_x, 0);
            cr.paint();
            margin_x+=star_width;
        }

        return false;
    }


}
}
