using Gtk;
using Gdk;
using Cairo;
using Granite;
using Granite.Widgets;


namespace Foto.Widgets{

    public class  ShortcutsPopUp: Gtk.Dialog{

    int shadow_blur = 15;
    int shadow_x    = 0;
    int shadow_y    = 2;
    double shadow_alpha = 0.3;
    private Granite.Drawing.BufferSurface buffer;
    private const string STYLESHEET = """
        * {
            -GtkDialog-action-area-border: 12px;
            -GtkDialog-button-spacing: 10px;
            -GtkDialog-content-area-border: 0;
        }
    """;

    private Gtk.Box column_1;
    private Gtk.Box column_2;

    public ShortcutsPopUp() {

        title = "";
        has_resize_grip = false;
        resizable = false;
        deletable = false; // Hide the window's close button when possible

        Granite.Widgets.Utils.set_theming (this, STYLESHEET, null,
                                           Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
        add_events (Gdk.EventMask.BUTTON_PRESS_MASK);
        // Set the default containers
        Box content_area = (Box)get_content_area();
        Box action_area = (Box)get_action_area();

        var draw_ref = new Gtk.Window ();
        draw_ref.get_style_context ().add_class (STYLE_CLASS_CONTENT_VIEW_WINDOW);

         // Apply DecoratedWindow's theming
        DecoratedWindow.set_default_theming (draw_ref);
        action_area.get_style_context ().add_class (STYLE_CLASS_CONTENT_VIEW);

        this.decorated = false; 
        this.set_visual (this.get_screen ().get_rgba_visual ());
        this.app_paintable = true;

        action_area.margin = 20;
        action_area.margin_bottom = 10;
        this.get_content_area ().margin = 30;
        this.get_content_area ().margin_top = 30;
        this.get_content_area ().margin_bottom = 10;

        var content_scrolled = new ScrolledWindow(null, new Adjustment(0, 0, 100, 1, 10, 0));
        content_scrolled.shadow_type = ShadowType.NONE;
        content_scrolled.hscrollbar_policy = PolicyType.NEVER;
        content_scrolled.vscrollbar_policy = PolicyType.AUTOMATIC;
        content_scrolled.height_request = 400;

        var content_hbox = new Box(Orientation.HORIZONTAL, 40);

        column_1 = new Box(Orientation.VERTICAL, 0);
        column_2 = new Box(Orientation.VERTICAL, 0);

        content_hbox.pack_start(column_1, true, true, 0);
        content_hbox.pack_start(column_2, true, true, 0);

        content_scrolled.add_with_viewport(content_hbox);

        // left and right padding
        content_hbox.pack_start(new Box(Orientation.VERTICAL, 0), false, false, 0);
        content_hbox.pack_end(new Box(Orientation.VERTICAL, 0), false, false, 0);
        content_area.pack_start(content_scrolled, true, true, 0);


        // Adjust sizes
        content_hbox.vexpand = false;
        content_hbox.hexpand = true;
        this.set_vexpand(true);
        this.set_hexpand(true);

            var w = -1;
            var h = -1;

            this.size_allocate.connect ( () => {
                if (this.get_allocated_width () == w && this.get_allocated_height () == h)
                    return;
                w = this.get_allocated_width ();
                h = this.get_allocated_height ();

                this.buffer = new Granite.Drawing.BufferSurface (w, h);

                this.buffer.context.rectangle (shadow_blur + shadow_x,
                    shadow_blur + shadow_y, w - shadow_blur*2 + shadow_x, h - shadow_blur*2 + shadow_y);
                this.buffer.context.set_source_rgba (0, 0, 0, shadow_alpha);
                this.buffer.context.fill ();
                this.buffer.exponential_blur (shadow_blur / 2);

                draw_ref.get_style_context ().render_activity (this.buffer.context, shadow_blur + shadow_x,
                    shadow_blur + shadow_y, w - shadow_blur*2 + shadow_x, h - shadow_blur*2 + shadow_y);

            });
            /*draw the buffer*/
            this.draw.connect ( (ctx) => {
                if (buffer == null)
                    return false;

                ctx.set_operator (Cairo.Operator.SOURCE);
                ctx.rectangle (0, 0, w, h);
                ctx.set_source_rgba (0, 0, 0, 0);
                ctx.fill ();

                ctx.set_source_surface (this.buffer.surface, 0, 0);
                ctx.paint ();

                return false;
            });


            /*allow moving the window*/
            this.button_press_event.connect ( (e) => {
                if (e.button == 1) {
                    this.begin_move_drag ((int) e.button, (int) e.x_root, (int) e.y_root, e.time);
                    return true;
                }
                return false;
            });

         this.key_press_event.connect (on_key_press);
    }

    public void add_shortcut(string key_val, string description,uint col /*1 or 2*/) {
        var label = new LLabel(description);
        label.set_alignment(0, 0.2f);
        var key_box = new Box(Orientation.HORIZONTAL, 0);
        key_box.pack_start(new KeyWidget(key_val), true, true, 0);
        key_box.pack_end (label, false, false, 0);

        if (col == 1) {
            column_1.pack_end(key_box, true, true, 0);
        }
        else {
            column_2.pack_end(key_box, true, true, 0);
        }


    }

    private bool on_key_press(EventKey e) {
        switch (e.keyval) {
            case Gdk.Key.Escape:
            case Gdk.Key.Super_L:
            case Gdk.Key.Super_R:
                this.set_visible(!this.get_visible());
                break;
            }
        return true;
    }

    }


    private class KeyWidget : Gtk.Box {

        string key_val = "";
        Gdk.Pixbuf key_pix;
    
        public KeyWidget (string key_val) {
            this.key_val = key_val;
            key_pix = new Pixbuf.from_file(Build.PKGDATADIR + "/key.png");
            this.width_request = 71;
            var label = new Label(key_val);
            this.add(label);
            this.draw.connect(draw_button);
            //this.height_request = 50;
        }


        public bool draw_button (Cairo.Context cr) {
            Gdk.cairo_set_source_pixbuf(cr,key_pix, 0, 0);
            cr.show_text("Hola");
            cr.paint();
            return false;
        }

    }
}
